function alertBox(parent,message)
{
    var w = loadXmlWidget("include/alertbox.xml", modal_root);
    if (!setModalFor(parent,w)) {
	w.destroy();
	return false;
    }
    w.titleWidget.multiLineText = message;
    
    w.grabInput(parent);
    w.cancel = function () { unsetModalFor(this.__page__); this.destroy(); }
    w.onHide = function () { unsetModalFor(this.__page__); this.destroy(); }
    w.acknowledged = function () { this.cancel(); }
    return true;
}

//
// object.ok() will be called if ok
// object.nok() will be called if user select no.
//
function questionBox(parent, message, object)
{
    var w = loadXmlWidget("include/questionbox.xml", modal_root);
    if (!setModalFor(parent,w)) {
	w.destroy();
	return false;
    }
    w.titleWidget.multiLineText = message;
    
    w.grabInput(parent);
    w.__done__ = object;
    w.ok = function () { var obj = this.__done__; unsetModalFor(this.__page__); this.destroy(); obj.ok(); }
    w.nok = function () { var obj = this.__done__; unsetModalFor(this.__page__); this.destroy(); obj.nok(); }
    
    w.onHide = function () { unsetModalFor(this.__page__); this.destroy(); }
    return true;
}

function selectNumber(parent, message, object, value, min, max, inc)
{
    var w = loadXmlWidget("include/enternumber.xml", modal_root);
    if (!setModalFor(parent, w)) {
	w.destroy();
	return false;
    }
    
    w.titleWidget.multiLineText = message;
    w.valueWidget.multiLineText = value;
    w.value = value;
    w.grabInput(parent);
    w.__done__ = object;

    w.minvalue = min;
    w.maxvalue = max;
    w.incvalue = inc;
    w.value = value;
    
    w.inc = function() {
	this.value += this.incvalue;
	if (this.value > this.maxvalue) {
	    this.value = this.minvalue;
	}
	this.valueWidget.multiLineText = this.value;
    }
    
    w.dec = function() {
	this.value -= this.incvalue;
	if (this.value < this.minvalue) {
	    this.value = this.maxvalue;
	}
	this.valueWidget.multiLineText = this.value;
    }
    
    w.ok = function () { 
    	var v = this.value;
    	var obj = this.__done__; 
    	unsetModalFor(this.__page__); 
    	this.destroy(); 
    	obj.ok(v); 
    }
    
    w.nok = function () { var obj = this.__done__; unsetModalFor(this.__page__); this.destroy(); obj.nok(); }
    
    w.onHide = function () { unsetModalFor(this.__page__); this.destroy(); }
    return true;
}

//
// It is assumed that path is under root.path (exception otherwise)
// paths must always start with a / and end with a /
function createFSItemForPath(root,path) {
    var item;
    var prepath,fsname,splitid;
    
    if (path == root.path) {
	return root;
    }

    if (path == "/") {
	throw new Error("path " + path + " not under root "+root+"\n");
    }
    
    // Remove the last part of and try again...
    item = new Object();
    splitid = path.lastIndexOf("/",path.length - 2);
    prepath = path.slice(0, splitid + 1);
    fsname = path.slice(splitid + 1, path.length - 1);
    
    item.display = displayBinaryPath(fsname);
    item.parent = createFSItemForPath(root, prepath)
    item.level = item.parent.level + 1;
    item.path = path;
    
    
    return item;
}

function removeFinalSlash(path)
{
    while((path.length > 1) && (path.substr(path.length-1) == "/")) {
	path = path.substr(0, path.length - 1);
    }
    return path;
}

// parent (widget) : the page on which the modal is to be displayed
// title (string)  : title of the dialog
// details (string) : additionnal text
// root (object) : root object (display, path, level)
// startitem (object) : initial directory FS item (see createFSItemForPath)
// filter (function) : function which filter items to be displayed in the FS Browser
// showName (boolean) : display or not an editbox with the file name (in this case, filter should only returns directories) 
// defaultName (string) : if showName is true, the default name to be displayed
// extension (string) : text to be appended to defaultName
// focushint (string) : undefined, "location", "name"
// objOk (string) : objOk.ok(path) will be called when done
function selectFileSystemItem(parent, title, details, root, startitem, filter, showName, defaultName, extension, focushint, objOk)
{
    var w;
    if (showName) {
	w = loadXmlWidget("include/newfileselector_dialog.xml", modal_root);
    } else {
    	w = loadXmlWidget("include/fileselector_dialog.xml", modal_root);
    }
    if (!setModalFor(parent,w)) {
	w.destroy();
	return false;
    }
    //w.titleWidget.multiLineText = message;
    w.grabInput(parent);
    w.onHide = function () { unsetModalFor(this.__page__); this.destroy(); }
    w.cancel = function () { unsetModalFor(this.__page__); this.destroy(); }
    w.useName = showName;
    if (showName) {
	w.textEditor.edittext = displayBinaryPath(defaultName);
    }
    w.extension = extension;
    
    root.level = 0;
    root.parent = undefined;
    w.title.text = title;
    w.details.multiLineText = details;
    w.fsBrowser.root = root;
    w.fsBrowser.itemFilter = filter;
    w.objOk = objOk;
    w.clicked = function() {
	var path = this.fsBrowser.currentPath;
	
	if (path == undefined) return;
	if (this.useName) {
	    var name = this.textEditor.edittext;
	    if ((name == undefined) || (name == "")) {
		return;
	    }
	    if (name.indexOf("/") != -1) {
		name = name.replace(/\//g,' ');
		this.textEditor.edittext = name;
		return;
	    }
	    print("encode binary path of "+name+"\n");
	    path = path + "/" + encodeBinaryPath(name) + this.extension;
	    path = path.replace(/\/+/g,'/');
	}
	
	this.cancel();
	print("selected path is "+path+"\n");
	this.objOk.ok(path);
    }
    
    if (startitem != undefined) {
	w.fsBrowser.currentFSItem = startitem;
    } else {
       w.fsBrowser.currentFSItem = root;
    }
    
    if (focushint == "location"){
	w.mainPanel.currentWidget = w.fsBrowser;
    } else if (showName && (focushint == "name" || defaultName == "")) {
	w.mainPanel.currentWidget = w.textEditor;
    } else {
	w.mainPanel.currentWidget = w.buttons;
    }
    
    return true;
}

// root: root item. Property for items are :
//	- display
//	- path
//	- subdirs (optional) => array...
// startpath (string) : directory path
// focushint (string) : undefined, "location", "name"
function selectFile(parent, title, details, root, startpath, startname, extension, focusHint, objOk)
{
    return selectFileSystemItem(	
	    parent, title, details,
	    root, createFSItemForPath(root, startpath),
	    fileselector_class.showOnlyDirectories,
	    true, startname, extension,
	    focusHint, 
	    objOk);
}

//
// root: root item. Property for items are :
//	- display
//	- path
//	- subdirs (optional) => array...
// focushint (string) : undefined, "location", "name"
// objOk.ok(path) will be called if a directory is selected
function selectAnyDirectory(parent, title, details, onlyDirectory, root, startitem, focusHint, objOk)
{
    return selectFileSystemItem(	
    		parent, title, details,
		root, startitem,
		(onlyDirectory ? fileselector_class.showOnlyDirectories : fileselector_class.showAllItems),
		false, "", "",
		focusHint, 
		objOk);
}

// Edit some text
// objOk.ok(path) will be called if the text is validated
function editText(parent, title, details, text, objOk)
{
    var w = loadXmlWidget("include/editbox_dialog.xml", modal_root);
    if (!setModalFor(parent,w)) {
	w.destroy();
	return false;
    }
    //w.titleWidget.multiLineText = message;
    w.grabInput(parent);
    w.onHide = function () { unsetModalFor(this.__page__); this.destroy(); }
    w.cancel = function () { unsetModalFor(this.__page__); this.destroy(); }
    
    w.title.text = title;
    w.details.multiLineText = details;
    w.objOk = objOk;
    
    w.validated = function () {
	var objok = this.objOk;
	var text = this.textEditor.edittext;
	this.cancel();
	objOk.ok(text);
    }
    
    /*w.textEditor.curs_move = function(i) {
	var p;
	
	p = this.cursorPos;

	p += i;
	if (p < 0) p = 0;
	if (p > utf8_length(this.multiLineText)) p = utf8_length(this.multiLineText);
	this.cursorPos = p;
    }
    
    w.textEditor.curs_begin = function() {
	this.cursorPos = 0;
    }
    
    w.textEditor.curs_end = function() {
	this.cursorPos = utf8_length(this.multiLineText);
    }
    
    w.textEditor.curs_suppr = function() {
	var txt,pos;
	txt = this.multiLineText;
	pos = this.cursorPos;
	
	this.multiLineText = utf8_substring(txt, 0, pos) + utf8_substring(txt,pos + 1,utf8_length(txt) - pos - 1);
    }

    w.textEditor.curs_del = function() {
	var txt,pos;
	txt = this.multiLineText;
	pos = this.cursorPos;
	if (pos == 0) return;
	this.multiLineText = utf8_substring(txt, 0, pos - 1) + utf8_substring(txt, pos, utf8_length(txt) - pos);
	this.cursorPos = pos - 1;
    }
    
    w.textEditor.key = function() {
	var ascii;
	ascii = getLastInputText();
	if (ascii != "") {
	    var txt,pos;
	    txt = this.multiLineText;
	    pos = this.cursorPos;
	    
	    this.multiLineText = utf8_substring(txt, 0, pos) + ascii + utf8_substring(txt,pos,utf8_length(txt) - pos);
	    this.cursorPos += utf8_length(ascii);
	}
    }
    
    w.textEditor.multiLineText = text;
    w.textEditor.cursorPos = utf8_length(text);
    w.textEditor.cursorOn = true;
    w.textEditor.cursorColor = 0; */
    w.textEditor.edittext = text;
    return true;
}

// autohide delay, in  1/40 ticks
function shader(w, autohide_delay, autohide_duration) {
    var sh = this;
    
    sh.widget = w;
    
    sh.autohide_delay = autohide_delay;
    sh.autohide_duration = autohide_duration;
    
    sh.timer = undefined;
    
    
    // status can be hidden, visible (ie disappearing)
    sh.status = "hidden";
    
    sh.permanentShow = function() {
	this.status = "visible";
	this.widget.visible = true;
	this.widget.transparency = 255;
	if (this.timer != undefined) {
	    this.timer.dispose();
	    this.timer = undefined;
	}
    }

    sh.show = function() {
	this.status = "visible";
	this.widget.visible = true;
	this.widget.transparency = 255;
	this.tick_left = this.autohide_delay + this.autohide_duration;
	
	if (this.timer == undefined) {
	    this.timer = newTimer();
	    this.timer.father = this;
	    this.timer.code = "this.father.timerTick();";
	}
	
	this.timer.monotone(40);
	this.timer.allowMissed = true;
    }
    
    sh.hide = function() {
	this.status = "hidden";
	this.widget.visible = false;
	this.tick_left = 0;
	if (this.timer != undefined) {
	    this.timer.dispose();
	    this.timer = undefined;
	}
    }
    
    sh.timerTick = function() {
	if (this.status == "hidden") {
	    this.timer.dispose();
	    this.timer = undefined;
	    return;
	}
	
	this.tick_left--;
	
	if (this.tick_left <= 0) {
	    this.widget.visible = false;;
	    this.timer.dispose();
	    this.timer = undefined;
	    this.status = "hidden";
	} else if (this.tick_left < this.autohide_duration) {
	    nvt = ((255 * this.tick_left) / this.autohide_duration);
	    this.widget.transparency = nvt;
	}
    }
    
    if (w.visible) {
	sh.show();
    }
    return sh;
}


//
// Various function which gives standard object sizes
//
var skin = new Object();
skin.buttonHeight = 30;
skin.buttonPanelHeight = 40;

skin.borderTop = 3;
skin.borderMiddleV = 2;
skin.borderBottom = 3;
skin.borderLeft = 3;
skin.borderMiddleH = 2;
skin.borderRight = 2;


skin.listItemHeight = 40;
skin.titleHeight = 30;

skin.roundShape = 15;

// Adjust a pixel count to a screen resolution.
// When pixel count would be inferior to 1, return 1.
function resol800x600(i) 
{
    var rslt;
    rslt = Math.floor((i * screen.height) / 600);
    if (rslt == 0) rslt = 1;
    return  rslt;
}

