// Inactivity modes can be 
//    nocheck : the inactivity is not checked
//    normal  : check inactivity in descent delay (some minutes)
//    wakeup  : check inactivity for some seconds (after wakeup)
//              if inactivity broke on elapse, system switches to normal mode

var inactivityWatcher;
var inactivityMode;

function initInactivityWatcher()
{
    inactivityWatcher = newInactivityWatcher();
    inactivityWatcher.enabled =  false;
    
    // This is called after the inactivity delay elapse
    inactivityWatcher.elapsed = function() {
	inactivityElapsed();
    }
    
    // This is called before handling an input that would break inactivity
    inactivityWatcher.broken = function() {
	setInactivityMode("normal");
    }
    
    setInactivityMode("nocheck");
    
    inactivityWatcher.restartOnBroke = false;
}

function setInactivityMode(mode)
{
    print("!!! setting inactivity mode to " + mode + " !!!\n");
    inactivityMode = mode;
    switch(inactivityMode)
    {
    case "wakeup":
	var d = getWakeupInactivityDuration();
	if (d != 0) {
	    inactivityWatcher.delay = d * 1000;
	    inactivityWatcher.restartOnBroke = false;
	    inactivityWatcher.enabled = true;
	    break;
	}
	// If 0 is configured, there is no short-poweroff timeout. Go back to normal now
    case "normal":
	// TODO : complete normal off mode :
	//     => xine playing should disable inactivity
	//     => external program running should disable inactivity
	//     => on inactivity, either turns off, or run an external program (like screen saver ?)
	// inactivityWatcher.delay = 20000;
	// Make it auto reset on user activity (broken will not be called)
	// inactivityWatcher.restartOnBroke = true;
	inactivityWatcher.enabled = false;
	break;
    case "nocheck":
    default:
	inactivityWatcher.enabled = false;
    }
    inactivityWatcher.reset();
}

function inactivityElapsed()
{
    if (inactivityMode == "wakeup") {
	print("!!! wakeup inactivity delay elapsed ! !!!\n");
	setInactivityMode("nocheck");
	// this will also set the inactivity mode to nocheck !
	mainmenu.terminate();
    } else {
	print("!!!! inactivity mode " + inactivityMode + " elapsed !!!\n");
	
	setInactivityMode("nocheck");
	mainmenu.terminate();
    }
}

initInactivityWatcher();

