// Number of consecutive short run before give up
var lirc_max_error = 10;
// Duration of a short run (maximum)
var lirc_minimum_duration = 2000;

var lirc_producer = undefined;
var lirc_restart_timer = undefined;
var lirc_start_time = 0;
var lirc_successive_error = 0;
var lirc_status_callback = new Array();


function lirc_get_helper_path()
{
    return getHelperPath("helpers/lirc/ostbox-lirc-helper");
}

function lirc_start() {

    if (lirc_producer != undefined) {
	// Kill current connection
	lirc_stop();
    }
    
    
    lirc_producer = newInputProducer();
    
    lirc_producer.onReply = function() {
	// set the current configuration
	if (this.lastReplyName == "connected") {
	    var xml = parseXml("<set_config_request><default-filter/></set_config_request>");
	    var root = xml.getRoot();
	    var filter = root.findChild(xmlNameFilter("default-filter"));
	    var key, val;
	    
	    // filter_gap
	    key = "lirc_filter_gap";
	    val = configHash.get(key);
	    if ((val == "") || (isNaN(parseInt(val)))) {
		val = 350;
		configHash.set(key, val);
	    }
	    filter.setAttribute("gap", val);

	    // Insert events
	    key = "lirc_filter_insert_event";
	    val = configHash.get(key);
	    if ((val != "0") && (val != "1")) {
		val = 1;
		configHash.set(key, val);
	    }
	    filter.setAttribute("insert-events", val);
	    
	    // Delay before repeat
	    key = "lirc_delay_before_repeat";
	    val = configHash.get(key);
	    if ((val == "") || (isNaN(parseInt(val)))) {
		val = 300;
		configHash.set(key, val);
	    }
	    filter.setAttribute("delay-before-repeat", val);
	    
	    // Repeat rate
	    key = "lirc_repeat_rate";
	    val = configHash.get(key);
	    if ((val == "") || (isNaN(parseInt(val))) || (val > 100)) {
		val = 3;
		configHash.set(key, val);
	    }
	    
	    if (val != 0) {
		filter.setAttribute("repeat-interval", Math.ceil(1000 / val));
	    }
	    
	    // Accelerated repeat rate
	    key = "lirc_repeat_rate_2";
	    val = configHash.get(key);
	    if ((val == "") || (isNaN(parseInt(val))) || (val > 100)) {
		val = 40;
		configHash.set(key, val);
	    }
	    if (val != 0) {
		filter.setAttribute("accel-interval", Math.ceil(1000 / val));
	    }
	    
	    // Duration of acceleration
	    key = "lirc_acceleration_duration";
	    val = configHash.get(key);
	    if ((val == "") || (isNaN(parseInt(val)))) {
		val = 4;
		configHash.set(key, val);
	    }
	    filter.setAttribute("accel-duration", 1000 * val);
	    
	    var splitter = new RegExp("[ ,;]+", "g");
	    var lirc_sources = configHash.get("lirc_socket").split(splitter);
	    for(var i = 0; i < lirc_sources.length; ++i) {
		var lirc_node = root.newChild("lirc");
		lirc_node.setText(lirc_sources[i]);
	    }
	    
	    var lirc_proxies = configHash.get("lirc_proxy").split(splitter);
	    for(var i = 0; i < lirc_proxies.length; ++i) {
		var lirc_node = root.newChild("proxy");
		lirc_node.setText(lirc_proxies[i]);
	    }
	    
	    print("Sending lirc configuration : "+xml.toString());
	    this.sendXmlRequest(xml, 5000, "set_config_result");
	} else if (this.lastReplyName == "set_config_result") {
	    print("lirc ready !\n");
	}
    }
    
    lirc_producer.onTimeout = function() {
	this.done();
    }
    
    lirc_producer.onDisconnect = function() {
	this.done();
    }
    
    // Automatic restart of lirc_producer
    lirc_producer.done = function() {
	this.disconnect();
	if (lirc_producer != this) return;
	
	lirc_producer = undefined;
	
	lirc_update_status();
	
	if (timeOfEvent - lirc_start_time > lirc_minimum_duration) {
	    // Don't consider that it is an error (normal termination, ???
	    lirc_successive_error = 0;
	    lirc_start();
	} else {
	    // Try again latter...
	    lirc_successive_error++;
	    
	    
	    if (lirc_restart_timer == undefined) {
		lirc_restart_timer = newTimer();
		lirc_restart_timer.code = "if (lirc_producer == undefined) lirc_start();";
	    }
	    
	    // Auto restart in 1, 2, 3, ...  second
	    lirc_restart_timer.singleStep(lirc_successive_error * 1000);
	}
    }
    
    var lirc_port = 12001;
    
    lirc_producer.setServerAutostartCommand(lirc_get_helper_path(), lirc_port)
    lirc_start_time = timeOfEvent();
    if (!lirc_producer.connect("127.0.0.1", lirc_port, "5000")) {
	lirc_producer.onDisconnect();
    }
    
    lirc_update_status();
}

function lirc_stop()
{
    lirc_successive_error = 0;
    
    if (lirc_restart_timer != undefined) {
	lirc_restart_timer.dispose();
	lirc_restart_timer = undefined;
    }
    
    if (lirc_producer == undefined) {
	return;
    }
    
    lirc_producer.disconnect();
    lirc_producer = undefined;
    
    lirc_update_status();
}

function lirc_reload()
{
    lirc_stop();
    lirc_start();
}

function lirc_disable()
{
    if (lirc_producer == undefined) {
	print("unable to disable lirc : no producer\n");
	return false;
    }

    // Disable output
    lirc_producer.sendXmlRequest("<disconnect_request/>", 10000, "disconnect_result");
    
    return lirc_producer.waitReply(true);
}

function lirc_enable()
{
    if (lirc_producer == undefined) {
	return false;
    }
    
    lirc_producer.sendXmlRequest("<connect_request/>", 10000, "<connect_result/>");
    return true;
}

function lirc_update_status()
{
    var i;
    for(i = 0; i < lirc_status_callback.length; ++i) {
	lirc_status_callback[i].lirc_status_changed(lirc_producer != undefined);
    }
}

// Callback is called whenever lirc status change
// callback_object.lirc_status_changed(bool) will be called
function lirc_listen_status(callback_object)
{
    lirc_status_callback[lirc_status_callback.length] = callback_object;
    callback_object.lirc_status_changed(lirc_producer != undefined);
}

