//{class:PanelFilter(Object)}
//
// This class should be used associated to a panel class
// It handles hidding and showing panelizable (actually button)
// depending on some condition.
//
// All childs of the Panel must be declared to the PanelFilter.
//
// The "condition" property is watched in childs and evaluated 
// in the scope of the PanelFilter object.
//
function PanelFilter(panel) {
    this.items = new Array();
    this.panel = panel;
	    
    //{func:addPanelizable(Panelizable)}
    // Add a Panelizable to this PanelFilter.
    // All Panelizable of the Panel must be added to the PanelFilter
    this.addPanelizable = function(w) {
	w.button_selector_pos = this.items.length;
	w.button_selector_visible = true;
	this.items.push(w);
	
	this.refreshCondition(w.button_selector_pos);
    }
    
    //{func:refreshCondition(int)}
    // Re-evaluate the visibility of a child, given its id
    this.refreshCondition = function(pos) {
	var w = this.items[pos];
	
	var isvisible = (!("condition" in w)) || (eval(w.condition));
	
	if (isvisible) {
	    w.visible = true;
	} else {
	    w.visible = false;
	    this.panel.jsDebug(" hiding "+pos+"\n");
	    this.panel.jsDebug(" current "+this.panel.panel_current_id+"\n");
	    if (this.panel.panel_current_id == pos) {
		var i;
		this.panel.jsDebug("hidden selected !\n");
		this.panel.debug();
		// We just hide the selection !
		for(i = pos -1; i > 0; --i) {
		    if (this.panel.panel_childs[i].visible) {
			this.panel.selectCurrentById(i);
			this.panel.jsDebug("selecting "+i+" !\n");
			return;
		    }
		}
		for(i = pos + 1; i < this.panel.panel_childs.length; ++i) {
		    if (this.panel.panel_childs[i].visible) {
			this.panel.selectCurrentById(i);
			this.panel.jsDebug("selecting "+i+" !\n");
			return;
		    }
		}
		// Sorry, loose selection !
		this.panel.jsDebug("loosed selection !\n");
		this.panel.selectCurrentById(undefined);
	    }
	}
	
	
	
	/*
	if (isvisible == w.button_selector_visible) return;
	
	w.button_selector_visible = isvisible;
	if (!isvisible) {
	    w.visible = false;
	    this.panel.removePanelizable(w);
	} else {
	    var i, realpos;
	    
	    realpos = 0;
	    for(i = 0; i < pos; ++i) {
		if (this.items[i].button_selector_visible) {
		    realpos++;
		}
	    }
	    w.visible = true;
	    this.panel.addPanelizableAt(w, realpos);
	}*/
    }
	
    //{func:refreshConditions()}
    // Re-evaluate the visibility of all childs
    this.refreshConditions = function() {
	var i;
	for(i = 0; i < this.items.length; ++i) {
	    this.refreshCondition(i);
	}
	if (this.panel.panel_current_id == undefined) {
	    for(i = 0; i < this.panel.panel_childs.length; ++i) {
		if (this.panel.panel_childs[i].visible) {
		    this.panel.jsDebug("selecting "+i+" !\n");
		    this.panel.selectCurrentById(i);
		    break;
		}
	    }
	}
	this.panel.refreshLayout();
    }
    
    //{func:importAllChilds()}
    // Declare all panels childs automatically (call this once!)
    this.importAllChilds = function() {
	var tmpChilds = this.panel.panel_childs.slice(0);
	for(i = 0; i < tmpChilds.length; ++i) {
	    debug("import : "+tmpChilds[i].title+"\n");
	    this.addPanelizable(tmpChilds[i]);
	}
    }
}
//{endclass}


//{class:FSPanelFilter(PanelFilter)}
//
// This class is a PanelFilter with specialised conditions 
// for testing against filesystem item (in browser).
function FSPanelFilter(panel, browser) {
    this.PanelFilter = PanelFilter;
    this.PanelFilter(panel);
    this.browser = browser;
    
    //{func:itemChanged()}
    //Find the current item and re-evaluate conditions
    this.itemChanged = function() {
	var item = this.browser.selectedItem();
	this.item = item;
	
	//{prop:hasCurrent} True if there is a current item
	this.hasCurrent = (item != undefined);
	
	//{prop:currentIsImage} True if current item is an image
	this.currentIsImage = (item != undefined) && ("type" in item) & (item.type == "image");
	//{prop:currentIsVideo} True if current item is a video
	this.currentIsVideo = (item != undefined) && ("type" in item) && (item.type == "video");
	//{prop:currentIsMusic} True if current item is a music file
	this.currentIsMusic = (item != undefined) && ("type" in item) && (item.type == "sound");
	//{prop:currentIsDirectory} True if current item is a directory
	this.currentIsDirectory = (item != undefined) && (item.isdir != 0);
	
	this.refreshConditions();
    }
    
    this.itemChanged();
    this.importAllChilds();
}
//{endclass}
