//
// page:
//	activate() => called when the page becomes visible
//	desactivate() => called when the page is hidden
//	Must be child of page_root.
//	They have only one modal dialog. 
//	When the page is hidden, its modal also is.
// modal:
//	onHide() => called when the modal is hidden because its father disappear
// 	cancel() => called when another modal needs to replace it.
//	
//

var page_root,modal_root;
var current_page;
var current_page_modal;

current_page = undefined;
current_page_modal = undefined;

page_root = newWidget();
page_root.father = screen;
page_root.width = screen.width;
page_root.height = screen.height;
page_root.alpha = 0;
page_root.visible = true;
page_root.setLayout(newFullLayout());
page_root.ready = true;

modal_root = newWidget();
modal_root.father = screen;
modal_root.visible = true;
modal_root.alpha = 0;
modal_root.width = screen.width;
modal_root.height = screen.height;
modal_root.setLayout(newFullLayout());
modal_root.ready = true;

function getModalFor(page)
{
    if (page == undefined) return undefined;
    if ("__modal__of__page__" in page) {
	return page.__modal__of__page__;
    }
    return undefined;
}

// Call this when the modal of the given page will be destroyed...
function unsetModalFor(page) 
{
    page.__modal__of__page__ = undefined;
    if (page == current_page) {
	current_page_modal = undefined;
    }
}

// Return true if possible, false otherwise (when page has an unclosable widget)
function setModalFor(page,modal)
{
    var actualModal;
    
    // First, try to cancel page modal
    actualModal = getModalFor(page);
    if (actualModal != undefined) {
	// cancel it ?
	
	if (!("cancel" in actualModal)) return false;
	debug("cancel current modal...\n");
	eval(actualModal.cancel());
	debug("ok\n");
	actualModal = getModalFor(page);
	if (actualModal != undefined) return false; 
    }
    
    page.__modal__of__page__ = modal;
    
    if (modal != undefined) {
	modal.__page__ = page;
	// Set the modal for page
	if (current_page == page) {
	    modal.visible = true;
	} else {
	    modal.visible = false;
	}
	modal.father = modal_root;
    }
    
    if (current_page == page) {
	debug("modal added to the current page !\n");
	current_page_modal = modal;
    }
    
    return true;
}

// set the current page. Always success
function setScreen(page)
{
    if (current_page == page) {
	return;
    }
    
    debug("switching page\n");
    if (current_page != undefined) {
	debug("hiding old page\n");
	if (current_page_modal != undefined) {
	    debug("hiding old modal\n");
	    if ("onHide" in current_page_modal) {
		current_page_modal.onHide();
	    }
	    if (current_page_modal != undefined) {
		current_page_modal.visible = false;
	    }
	}
	current_page.visible = false;
	current_page.desactivate();
	current_page = undefined;
    }
    
    current_page = page;
    if (current_page != undefined) {
	current_page.visible=true;
	if ("activate" in current_page) {
	    current_page.activate();
	}
    }
}

function isCurrentScreen(page)
{
    return current_page == page;
}

function getCurrentScreen()
{
    return current_page;
}
