
/*
 Helpers functions
*/
// Update a delete button for the given item
// (make it an interrupt/cancel/...) depending on status
function updateDeleteButtonForRecord(page, hideProgress, bton, item)
{
    bton.onclick = "this.perform()";
    bton.record_obj = item;
    bton.perform = function() {};
    
    bton.page = page;
    bton.hideProgress = hideProgress;
    
    if (item == undefined) {
	state = undefined;
    } else {
	state = item.state;
    }
    
    // We have three buttons :
    // pending : 1 - Apply, 2 - Delete, 3 - Back
    // running : 1 - Apply, 2 - Interrupt, 3 - Back
    // done    : 1 - Apply, 2 - Delete, 3 - Back
    if (state == "pending") {
	bton.title = i18n("Cancel record");
	bton.perform = function() {
	    print("cancel record.\n");
	    var obj = newEmptyRecord();
	    obj.id = this.record_obj.id;
	    obj.state = "canceled";
	    updateRecord(obj, this.page, this.hideProgress);
	}
    } else if (state == "running") {
	bton.title = i18n("Interrupt record");
	bton.perform = function() {
	    var obj = newEmptyRecord();
	    obj.id = this.record_obj.id;
	    obj.state = "interrupted";
	    updateRecord(obj, this.page, this.hideProgress);
	}
    } else {
	bton.title = i18n("Delete record");
	if (item == undefined) {
	    bton.perform = function() {};
	} else {
	    bton.perform = function() {
		print("delete record.\n");
		var obj = newEmptyRecord();
		obj.id = this.record_obj.id;
		
		var ackObj = new Object();
		ackObj.obj = obj;
		ackObj.page = this.page;
		ackObj.nok = function() {}
		ackObj.ok = function() {
		    supprRecord(this.obj, this.page, false);
		}
		questionBox(this.page, i18n("Do you really want to delete $1 ?", this.record_obj.title), ackObj); 
	    }
	}
    }
}

function getRecordIcon(record)
{
    // Record is pending and has no tunerid
    if (record.state == "pending") {
	if ((record.tuner_id == "") || (record.tuner_id == undefined)) {
	    // ?
	    return "icons/icon_unchanged.svgz.png";
	}
	// clock
	return "icons/icon_clock.svgz.png";
    }
    
    // OK
    if (record.state == "done") return "icons/icon_enabled.svgz.png";
    // Running
    if (record.state == "running") return "icons/icon_cache.svgz.png";
    // disabled
    if (record.state == "canceled") return "icons/icon_disabled.svgz.png";
    
    // KO (!)
    // if ((record.state == "missed") || (record.state == "interrupted")) 
    return "icons/icon_error.svgz.png"; 
}

// List channels which can be recorded.
function getChannels(arrAccelerator, arrId, arrDisplay)
{
    // Search for tuners that have a helper tag
    // Search for frequency which have this tag
    if (zapping.config == null) return;
    var view = zapping.config.find("/channel");
    for(i = 0; i < view.getItemCount(); ++i) {
	var channel = view.getItem(i);
	
	var shortcut = channel.get("shortcut");
	var display = channel.get("display");
	var id = channel.get("id");
	
	arrAccelerator.push(shortcut);
	arrId.push(id);
	arrDisplay.push(display);
    }
    
    return arrId.length > 0;
}

/* Dialogs
*/

function popupRecordEditor(page)
{
    var arrAccel, arrValue, arrDisplay;
    
    arrAccel = new Array();
    arrValue = new Array();
    arrDisplay = new Array();
    
    if (!getChannels(arrAccel, arrValue, arrDisplay)) {
    	return undefined;
    }
    
    var w = loadXmlWidget("plugins/recorder/edit_record.xml", modal_root);
    if (!setModalFor(page,w)) {
	w.destroy();
	return undefined;
    }
    w.grabInput(page);
    w.cancel = function () { unsetModalFor(this.__page__); this.destroy(); }
    w.onHide = function () { unsetModalFor(this.__page__); this.destroy(); }
    
    w.channel_selector.itemAccelerator = arrAccel;
    w.channel_selector.itemValue = arrValue;
    w.channel_selector.itemDisplay = arrDisplay;
    w.channel_selector.value = arrValue[0];
    
    w.loadTitleFrom = function(obj) {
	this.title_editor.edittext = obj.title;
    }
    
    w.loadChannelFrom = function(obj) {
	this.channel_selector.value = obj.channel;
    }
    
    w.loadStartTimeFrom = function(obj) {
	var timestamp = epochToTimestamp(obj.start_time);
	print("timestamp is "+timestamp+"\n");
	this.date_selector.value = timestamp.substr(2,6);
	this.start_hour_selector.value = timestamp.substr(8,2);
	this.start_min_selector.value = timestamp.substr(10,2);
	this.start_sec_selector.value = timestamp.substr(12,2);
    }
    
    w.loadDurationFrom = function(obj) {
	this.hour_selector.value = Math.floor(obj.duration / 3600);
	this.min_selector.value = Math.floor(obj.duration  / 60) % 60;
	this.sec_selector.value = obj.duration % 60;
    }
    
    w.loadFromObj = function(obj) {
	if (obj.title != undefined) this.loadTitleFrom(obj);
	if (obj.channel != undefined) this.loadChannelFrom(obj);
	if (obj.start_time != undefined) this.loadStartTimeFrom(obj);
	if (obj.duration != undefined) this.loadDurationFrom(obj);
    }
    
    w.setToObj = function(obj) {
	obj.title = this.title_editor.edittext;
	obj.channel = this.channel_selector.value;
	
	var ymd,y,m,d;
	var tmpdate = newDate();
	ymd = this.date_selector.value;
	y = parseInt(ymd.substr(0,2), 10);
	m = parseInt(ymd.substr(2,2), 10);
	d = parseInt(ymd.substr(4,2), 10);
	tmpdate.local = true;
	tmpdate.setDate(y + 100, m - 1, d, 
	    		this.start_hour_selector.value, 
			this.start_min_selector.value, 
			this.start_sec_selector.value);
	
	obj.start_time = tmpdate.epoch;
	
	obj.duration = this.hour_selector.value * 3600 + this.min_selector.value * 60 + this.sec_selector.value * 1;
    }
    
    return w;
}

function createNewRecordDialog(page, initobj)
{
    if (configHash.get("recorder_directory") == "") {
	alertBox(page, i18n("Please configure recorder files directory"));
	return;
    }
    var w = popupRecordEditor(page);
    if (w == undefined) return;
    
    w.title_widget.text = i18n("New record");
    
    w.icon.background = "icons/icon_clock.svgz.png";
    
    w.title_editor.edittext = i18n("New record");
    
    var nt = nowTimestamp();
    
    w.date_selector.value = nt.substr(2,6);
    w.start_hour_selector.value = nt.substr(8,2);
    w.start_min_selector.value = nt.substr(10,2);
    w.start_sec_selector.value = "00";
    
    w.perform = function() {
	// Push the record to the recorder
	var obj = newEmptyRecord();
	var page = this.__page__;
	this.setToObj(obj);
	
	this.cancel();
	
	// FIXME : put this somewhere else (?)
	obj.target_directory = configHash.get("recorder_directory");
	if (obj.target_directory != "") {
	    affectFrequencyToRecord(obj);
	    
	    createRecordFor(obj, page, false);
	}
    }
    
    if (initobj != undefined) {
	w.loadFromObj(initobj);
	w.ok_bton.takeFocus();
    }
}

function editRecordDialog(page, initobj)
{
    var w = popupRecordEditor(page);
    if (w == undefined) return;
    
    // Add a button
    w.interrupt_bton = newWidgetInstance("button", w.back_bton.father);
    w.interrupt_bton.onclick = "this.father.father.interrupt()";
    w.interrupt_bton.panel = w.back_bton.panel;
    w.interrupt_bton.cursorHint = w.back_bton.cursorHint;
    w.interrupt_bton.title = "delete";
    
    w.back_bton.cursorHint = (1 + parseInt(w.back_bton.cursorHint.substr(0, 1), 10)) + w.back_bton.cursorHint.substr(1);
    
    w.title_widget.text = i18n("Edit record");
    
    w.obj = initobj.duplicate();
    w.icon.background = getRecordIcon(w.obj);
    
    w.cancel = function () { this.watcher.forget(); unsetModalFor(this.__page__); this.destroy(); }
    w.onHide = function () { this.watcher.forget(); unsetModalFor(this.__page__); this.destroy(); }
    
    w.watcher = newRecordIDWatcher(initobj.id);
    w.watcher.father = w;
    
    w.refreshButton = function() 
    {
	// Refresh the buttons depending on watcher.currentItem
	updateDeleteButtonForRecord(this.__page__, false, this.interrupt_bton, this.watcher.item);
    }
    
    w.watcher.updated = function()
    {
	var currentItem = this.father.obj;
	// Report status changed
	if (this.item == undefined) {
	    this.father.cancel();
	    return;
	}
	
	this.father.icon.background = getRecordIcon(this.item);
	if (item.title != currentItem.title) {
	    currentItem.title = item.title;
	    this.father.loadTitleFrom(currentItem);
	}
	
	if (item.channel != currentItem.channel) {
	    currentItem.channel = item.channel;
	    this.father.loadChannelFrom(currentItem);
	}
	
	if (item.start_time != currentItem.start_time) {
	    currentItem.start_time = item.start_time;
	    this.father.loadStartTimeFrom(currentItem);
	}
	
	if (item.duration != currentItem.duration) {
	    currentItem.duration = item.duration;
	    this.father.loadDurationFrom(currentItem);
	}
	this.father.refreshButton();
    }
    
    w.perform = function() {
	// Push the record to the recorder
	var obj = this.obj;
	var changeObj = newEmptyRecord();
	
	changeObj.id = this.obj.id;
	this.setToObj(changeObj);

	var page = this.__page__;
	this.cancel();	

	// report progress (hide progress = false)
	updateRecord(changeObj, page, false);
    }
    
    w.loadFromObj(initobj);
    w.refreshButton();
    w.ok_bton.takeFocus();
    
}


/* Browser stuffs
*/
browser.createWidgetFor = function(item)
{
    var date = newDate();
    print ("loading child\n");
    nvchild = loadXmlWidget("plugins/recorder/item.xml", null);
    nvchild.icon.background = getRecordIcon(item);
    nvchild.title.text = item.title;
    nvchild.channel.text = item.frequency;
    date.local = true;
    date.epoch = item.start_time;
    nvchild.date.text = date.day +"/"+(date.month+1)+"/"+(1900+date.year)
    nvchild.hour.text = date.hour+":"+date.minute+":"+date.second;
    return nvchild;
}

browser.sortFunction = function(a, b)
{
    return cmpEpoch(a.start_time, 0, b.start_time);
}

browser.updateStatusBar = function()
{
}

browser.onSelectionChanged = "this.selectedChanged()";

browser.selectedChanged = function() {
    updateDeleteButtonForRecord(recorder, false, delete_bton, browser.selectedItem());
}

function editCurrent()
{
    var item = browser.selectedItem();
    if (item == undefined) return;
    
    editRecordDialog(recorder, item.duplicate());
}

function clickCurrent()
{
    var item = browser.selectedItem();
    if (item == undefined) return;
    
    if (item.path != "") {
	video_main.play(item.path);
    } else if ((item.state == "running") || (item.state == "done") || (item.state = "interrupted")) {
	alertBox(recorder, i18n("No video was produced for this record"));
    } else {
	editRecordDialog(recorder, item.duplicate());
    }
}


var view = newRequest();

view.setFilter(function(item) {
    //return item.state == "pending";
    return true;
});

view.addRecord = function(item) {
    if (browser.findItem(item) != undefined) {
	print("resort item.\n");
	browser.resortItem(item);
	browser.recreateWidgetForItem(item);
	if (item == browser.selectedItem()) {
	    browser.selectedChanged();
	}
    } else {
	browser.addItem(item);
    }
};

view.updateRecord = function(item) {
    print("resort item.\n");
    browser.resortItem(item);
    browser.recreateWidgetForItem(item);
    if (browser.selectedItem() == item) {
	browser.selectedChanged();
    }
}

view.removeRecord = function(item) {
    browser.removeItem(item);
}



// Change the view to filter only some specific items.
function selectRecords(arr)
{
    // Nothing for the moment
}

function activate() {
    this.currentWidget = this.browser;
    
    if (zapping.config == null) {
	config_main.popupTVSettings();
	mainmenu.redirect(this, config_main);
	return;
    }
}

function desactivate() {
}


