// Recorder javascript code

var recordsRequests = newWeakReference();
var records = new Array();

var recorderPort = 12000;
var recordMonitor = undefined;
var recordMonitorWatchdog = undefined;
var recorderOrders = new Array;


function startRecorderClient(reportPage, hideProgress)
{
    var client = newXmlClient();
    
    client.setServerAutostartCommand(getHelperPath("helpers/recorder/ostbox-recorder-helper"), recorderPort);
    client.status = "connecting";

    client.reportPage = reportPage;
    client.hideProgress = hideProgress;
    client.modal = undefined;
    
    client.start = function() {
	// connect to the helper. Give 5 seconds for timeout
	return this.connect("127.0.0.1",recorderPort, 5000);
    }
    
    client.reportProgress = function(message) {
    	if (this.modal != undefined) {
	    this.reportClose();
	}

	if (this.reportPage == undefined) return;
	if (this.hideProgress) return;
		
	var w = loadXmlWidget("include/progress_widget.xml", modal_root);
	if (w == undefined) return;
	
	if (!setModalFor(this.reportPage, w)) {
	    w.destroy();
	    return;
	}

	w.grabInput(this.reportPage);
	w.client = this;
	w.cancel = function() { 
	    this.client.modal = undefined;
	    this.client = undefined;
	    unsetModalFor(this.__page__);
	    this.destroy();
	}
	
	// Don't hide : w.onHide = function() { unsetModalFor(this.__page__); this.destroy(); }
	w.title = message;

	this.modal = w;
    }
    
    client.reportError = function(message) {
    	if (this.modal != undefined) {
    	    this.reportClose();
    	}
	
	if (this.reportPage == undefined) return;
    	alertBox(this.reportPage, message);
    }
    
    client.reportClose = function() {
    	if (this.modal == undefined) return;
    	this.modal.cancel();
    	// this.modal = undefined;
    }
    
    return client;
}


// Return an object which will asynchronously check for next possible sleep time (epoch)
// The following method will be called when result is available :
//	error() => unable to get information
//	result(when) => when is "no", "forever", an epoch
// On error, return undefined
function getWakeupPossibilityChecker()
{
    var client = startRecorderClient(undefined);
    
    client.onDisconnect = function() {
    	this.error();
    }
    
    client.onTimeout = function() {
    	this.disconnect();
    	this.error();
    }
    
    client.onReply = function() {    
        if (this.status == "connecting") {
            /*this.status = "locking";
            rslt = this.SendXmlRequest("<lock_request/>", 5000, "max_sleep_result");
        } else if (this.status == "locking") {*/
            this.status = "talking";
            rslt = this.sendXmlRequest("<max_sleep_request/>", 5000, "max_sleep_result");
        } else if (this.status == "talking") {
	    var xmlReply = this.getLastReply();
	    var when = xmlReply.getRoot().getAttribute("when");
	    print("may sleep : when = " + when + "\n");
	    this.result(when);
        } else {
    	    this.disconnect();
            this.error();
        }
    }
    
    if (!client.start()) {
    	return undefined;
    }
    return client;
}

function startRecordMonitor()
{
    var client = startRecorderClient(undefined);
    
    client.onReply = function() {
	var rslt = false;
	print("on reply called\n");
	if (this.status == "connecting") {
	    this.status = "initiating";
	    rslt = this.sendXmlRequest("<monitor_request/>", 5000, "monitor_result");
	} else if(this.status == "initiating") {
	    this.status = "listening";
	    rslt = true;
	}
	
	if (!rslt) {
	    print("Error talking to record helper, in status "+this.status+"\n");
	    this.disconnect();
	    recordMonitor = undefined;
	}
    }
    
    client.onNotification = function()
    {
	print("on notification called\n");
	// Load the notification to an object
	// Get the id
	if (this.lastNotificationName == "change_notif") {
	    var xmlNotification = this.getLastNotification();
	    var obj = loadRecordFromXmlNode(xmlNotification.getRoot());
	    var oldobj = undefined;;
	    var allRequests, i, index;
	    if (obj == undefined) return;
	    
	    item = undefined;
	    index = undefined;
	    for(i = 0; i < records.length; ++i) {
		if (records[i].id == obj.id) {
		    item = records[i];
		    index = i;
		    break;
		}
	    }
	    // Iterate all requests
	    allRequests = recordsRequests.content();
	    print("number of request is "+allRequests.length+"\n");
	    if (item == undefined) {
		print("item was not found in the cache\n");
	    } else {
		print("item was in the cache at "+index+"\n");
	    }
	    
	    print("obj.state = "+obj.state+"\n");
	    // Handle deletion now
	    if (obj.state == "deleted") {
		print("deleted item\n");
		if (item == undefined) return;
		
		// Remove object from record list
		records.splice(index, 1);
		
		for(i = 0; i < allRequests.length; ++i) {
		    if (allRequests[i].filter(item)) {
			print("removing from request at "+i+"\n");
			allRequests[i].removeRecord(item);
		    }
		}
		return;
	    }
	    
	    // New item ?
	    if (item == undefined) {
		print("new item\n");
		item = obj;
		records.push(item);
		for(i = 0; i < allRequests.length; ++i) {
		    if (allRequests[i].filter(item)) {
			allRequests[i].addRecord(item);
		    }
		}
		return;
	    }
	    
	    // Just a change in item
	    print("item change\n");
	    oldobj = item.duplicate();
	    obj.copyTo(item);
	    for(i = 0; i < allRequests.length; ++i) {
		var oldok, newok;
		oldok = allRequests[i].filter(oldobj);
		newok = allRequests[i].filter(item);
		
		if (oldok && newok) {
		    allRequests[i].updateRecord(item);
		} else if (oldok && (!newok)) {
		    allRequests[i].removeRecord(item);
		} else if (newok && (!oldok)) {
		    allRequests[i].addRecord(item);
		}
	    }
	    
	    
	    return;
	}
	print("Unhandled notification from record-helper : "+ this.lastNotificationName+", in status "+this.status+"\n");
    }
    
    client.onDisconnect = function()
    {
	print("Disconnected from record-helper in status "+this.status+"\n");
	recordMonitor = undefined;
    }
    
    client.onTimeout = function()
    {
	print("Timeout from record-helper in status "+this.status+"\n");
	this.disconnect();
	recordMonitor = undefined;
    }
    
    if (client.start()) {
	recordMonitor = client;
    } else {
	print("recorder client could not start.\n");
    }
}

function startRecordMonitorWatchdog()
{
    recordMonitorWatchdog = newTimer();
    recordMonitorWatchdog.code = "this.checkStatus()";
    recordMonitorWatchdog.checkStatus = function()
    {
	if (recordMonitor == undefined) {
	    print("record monitor was stopped. restarting.\n");
	    startRecordMonitor();
	}
    }
    // Check monitor
    recordMonitorWatchdog.monotone(10000);
    recordMonitorWatchdog.allowMissed = true;
}

function affectFrequencyToRecord(obj)
{
    // Just find one for the moment (conflict and multi tuner spanning will come later)
    var view = zapping.config.find("/channel[id]/frequency", obj.channel);
    var i, j;
    for(i = 0; i < view.getItemCount(); ++i ){
	var frequency_ref = view.getItem(i);
	tunerid = frequency_ref.get("tuner-id");
	frequencyid = frequency_ref.get("frequency-id");
	
	var helper = zapping.config.findFirst("/tuner[id]/helper",tunerid);
	
	if (helper != null) {
	    // Ok, we have all information
	    var port = helper.get("port");
	    var start_command = getHelperPath("helpers/media/ostbox-media-helper");
	    
	    obj.frequency = frequencyid;
	    obj.tuner_port = port;
	    obj.tuner_cmd = start_command;
	    obj.tuner_id = tunerid;
	    return;
	}
    }
    obj.frequency = "";
    obj.tuner_port = "";
    obj.tuner_cmd = "";
    obj.tuner_id = "";
}

function newEmptyRecord()
{
    var rslt = new Object();
    rslt.id = undefined;
    rslt.target_directory = undefined;
    rslt.title = undefined;
    rslt.path = undefined;
    rslt.channel = undefined;
    rslt.frequency = undefined;
    rslt.tuner_port = undefined;
    rslt.tuner_cmd = undefined;
    rslt.tuner_id = undefined;
    rslt.start_time = undefined;
    rslt.duration = undefined;
    rslt.state = undefined;
    
    rslt.duplicate = function()
    {
	var rslt = newEmptyRecord();
	this.copyTo(rslt);
	return rslt;
    }
    
    rslt.copyTo = function(obj)
    {
	obj.id = this.id;
	obj.target_directory = this.target_directory;
	obj.title = this.title;
	obj.path = this.path;
	obj.frequency = this.frequency;
	obj.channel = this.channel;
	obj.tuner_port = this.tuner_port;
	obj.tuner_cmd = this.tuner_cmd;
	obj.tuner_id = this.tuner_id;
	obj.start_time = this.start_time;
	obj.duration = this.duration;
	obj.state = this.state;
	
    }
    
    rslt.saveToXmlNode = function(node)
    {
	if (this.id != undefined) node.setAttribute("id", this.id);
	if (this.target_directory != undefined) node.setAttribute("target_directory", this.target_directory);
	if (this.title != undefined) node.setAttribute("title", this.title);
	if (this.path != undefined) node.setAttribute("path", this.path);
	if (this.channel != undefined) node.setAttribute("channel", this.channel);
	if (this.frequency != undefined) node.setAttribute("frequency", this.frequency);
	if (this.tuner_port != undefined) node.setAttribute("tuner_port", this.tuner_port);
	if (this.tuner_cmd != undefined) node.setAttribute("tuner_cmd", this.tuner_cmd);
	if (this.tuner_id != undefined) node.setAttribute("tuner_id", this.tuner_id);
	if (this.start_time != undefined) node.setAttribute("start_time", this.start_time);
	if (this.duration != undefined) node.setAttribute("duration", this.duration);
	if (this.state != undefined) node.setAttribute("state", this.state);
    }

    
    return rslt;
}

function loadRecordFromXmlNode(node)
{
    var rslt = newEmptyRecord();
    
    print("node name is "+node.getName()+"\n");
    rslt.id = node.getAttribute("id");
    rslt.target_directory = node.getAttribute("target_directory");
    rslt.title = node.getAttribute("title");
    rslt.path = node.getAttribute("path");
    rslt.channel = node.getAttribute("channel");
    rslt.frequency = node.getAttribute("frequency");
    rslt.tuner_port = node.getAttribute("tuner_port");
    rslt.tuner_cmd = node.getAttribute("tuner_cmd");
    rslt.tuner_id = node.getAttribute("tuner_id");
    rslt.start_time = node.getAttribute("start_time");
    rslt.duration = node.getAttribute("duration");
    rslt.state =  node.getAttribute("state");
    
    
    print("id is "+rslt.id+"\n");
    if (rslt.id == "") {
	return undefined;
    }
    return rslt;
}


function forgetOrder(obj)
{
    var i;
    for(i = 0; i < recorderOrders.length; ++i) {
	if (recorderOrders[i] == obj) {
	    recorderOrders.splice(i, 1);
	    return;
	}
    }
    print("unable to remove object from recorderOrders !!!!!!!!!!!!!!!!!!!!!!!");
}

function supprRecord(record, reportPage, hideProgress)
{
    var client = startRecorderClient(reportPage, hideProgress);
    
    client.record = newEmptyRecord();
    client.record.id = record.id;
    
    client.onReply = function() {
	var rslt = false;
	if (this.status == "connecting") {
	    this.status = "deleting";
	    var xml = parseXml("<delete_request/>");
	    this.record.saveToXmlNode(xml.getRoot());
	    rslt = this.sendXmlRequest(xml.toString(), 30000, "delete_result");
	} else if (this.status == "deleting") {
	    // delete is always possible
	    print("deleting succeeded\n");
	    this.disconnect();
	    forgetOrder(this);
	    this.reportClose();
	    return;
	}
	if (!rslt) {
	    this.disconnect();
	    forgetOrder(this);
	    this.reportError(i18n("Failed to delete record schedule"));
	}
    }
    
    client.onDisconnect = function()
    {
	this.disconnect();
	forgetOrder(this);
	this.reportError(i18n("Failed to delete record schedule"));
    }
    
    client.onTimeout = function()
    {
	this.disconnect();
	forgetOrder(this);
	this.reportError(i18n("Failed to delete record schedule"));
    }
    
    if (!client.start()) {
    	client.reportError(i18n("Unable to delete record schedule"));
    	return;
    }
    client.reportProgress(i18n("Deleting record schedule"));
    recorderOrders.push(client);
}

// If page is not undefined, a progression dialog will popup on the page
function createRecordFor(record, reportPage, hideProgress)
{
    var client = startRecorderClient(reportPage, hideProgress);
    
    client.record = record.duplicate();
    
    client.onReply = function() {
	var rslt = false;
	if (this.status == "connecting") {
	    this.status = "creating";
	    var xml = parseXml("<create_request/>");
	    this.record.saveToXmlNode(xml.getRoot());
	    rslt = this.sendXmlRequest(xml.toString(), 10000, "create_result");
	} else if (this.status == "creating") {
	    var xmlReply = this.getLastReply();
	    var id = xmlReply.getRoot().getAttribute("id");
	    
	    // We have an id.
	    // FIXME : we should be able to send the id back for autoselection
	    if (id != "") {
		print("create succeeded\n");
		this.disconnect();
		
		forgetOrder(this);
		
		this.reportClose();
		return;
	    }
	    
	    // No id : problem occured
	    rslt = false;
	}
	if (!rslt) {
	    this.disconnect();
	    forgetOrder(this);
	    client.reportError(i18n("Failed to save new record schedule"));
	}
    }
    
    client.onDisconnect = function()
    {
	this.disconnect();
	forgetOrder(this);
	this.reportError(i18n("Failed to save record schedule"));
    }
    
    client.onTimeout = function()
    {
	this.disconnect();
	forgetOrder(this);
	this.reportError(i18n("Failed to save record schedule"));
    }
    
    
    if (!client.start()) {
    	client.reportError(i18n("Failed to save new record schedule"));
    	return;
    }
    recorderOrders.push(client);
    client.reportProgress(i18n("Saving new record schedule"));    
}


function updateRecord(record, reportPage, hideProgress)
{
    var client = startRecorderClient(reportPage, hideProgress);
    
    client.record = record.duplicate();
    
    client.onReply = function() {
	var rslt = false;
	if (this.status == "connecting") {
	    this.status = "updating";
	    var xml = parseXml("<update_request/>");
	    this.record.saveToXmlNode(xml.getRoot());
	    rslt = this.sendXmlRequest(xml.toString(), 10000, "update_result");
	} else if (this.status == "updating") {
	    var xmlReply = this.getLastReply();
	    var ok = xmlReply.getRoot().getAttribute("ok");
	    
	    // We have a ok ?
	    if (ok == "1") {
		print("update succeeded\n");
		this.disconnect();
		
		forgetOrder(this);
		
		this.reportClose();
		return;
	    }
	    
	    // No id : problem occured
	    rslt = false;
	}
	if (!rslt) {
	    this.disconnect();
	    forgetOrder(this);
	    this.reportError(i18n("Failed to update record schedule"));
	}
    }
        
    client.onDisconnect = function()
    {
	this.disconnect();
	forgetOrder(this);
	this.reportError(i18n("Failed to update record schedule"));
    }
    
    client.onTimeout = function()
    {
	this.disconnect();
	forgetOrder(this);
	this.reportError(i18n("Failed to update record schedule"));
    }

    if (!client.start()) {
    	client.reportError(i18n("Failed to update record schedule"));
    	return;
    }
    recorderOrders.push(client);
    client.reportProgress(i18n("Updating record schedule"));
}



// This functions create a new empty request ID.
// Request has the following interface :
//	filter(item)		Check if the record is to be returned by the request.
//				Must be overriden (false by default)
//	setFilter(func)		Change the filter function and apply it.
//	refreshFilter()		Call this when result of filter(item) may have changed
//				During this function, addRecord may be called for item already present.
//	addRecord(item)		called when an item is to be added to the resultset
//	removeRecord(item)	called when an item is to be removed from the resultset
//      updateRecord(item)	called when an item has changed
//	forget()		call this when no more update are required.
//				otherwise, update are received until garbage collection !
function newRequest()
{
    var request = newWeakObject();
    
    
    // By default, request does not accept items
    request.filter = function(item) {
	return false;
    }
    
    request.forget = function() {
	recordsRequests.removeObject(this);
    }
    
    request.setFilter = function(f) {
	var i, j, record;
	var past, futur;
	
	this.newFilter = f;
	
	for(i = 0; i < records.length; ++i) {
	    record = records[i];
	    past = this.filter(record);
	    futur = this.newFilter(record);
	    if (past && (!futur)) {
		this.removeRecord(record);
	    }
	    if ((!past) && (futur)) {
		this.addRecord(record);
	    }
	}
	this.filter = this.newFilter;
	delete this.newFilter;
    }
    
    request.refreshFilter = function() {
	var i, record;
	for(i = 0; i < records.length; ++i) {
	    record = records[i];
	    if (this.filter(record)) {
		this.addRecord(record);
	    } else {
		this.removeRecord(record);
	    }
	}
    }
    
    request.addRecord = function(item) {}
    request.removeRecord = function(item) {}
    request.updateRecord = function(item) {}
    recordsRequests.addObject(request);
    return request;
}

function newRecordIDWatcher(id)
{
    var recordWatcher = newRequest();
    
    recordWatcher.id = id;
    recordWatcher.item = undefined;
    
    recordWatcher.updated = function() {};
    
    recordWatcher.addRecord = function(item) {
	this.item = item.duplicate();
	this.updated();
    };
    
    recordWatcher.removeRecord = function(item) {
	this.item = undefined;
	this.updated();
    };
    
    recordWatcher.updateRecord = function(item) {
	this.item = item.duplicate();
	this.updated();
    };
    
    recordWatcher.setFilter(function(item) {
	return item.id == this.id;
    });
    
    return recordWatcher;
}

// Create a record request on non empty record, with two additionnal functions:
// refreshFilter must be called manually when parameters are changed
// min : minimal timestamp (YYYYMMDDHHMMSS)
// max : maximal timestamp (YYYYMMDDHHMMSS)
// unterminatedOnly : returns only records which are not terminated (running or pending)
// channel : channel id
// items : list of matching records
// must explicitely call refreshFilter on min,max update
// findRecords(min, max) return parts of the matching records which cover the given interval. Undefined if none, array otherwise
// updated() called each time a record change have occured
function newTimestampRequest()
{
    var recordWatcher = newRequest();
    
    recordWatcher.minTimestamp = "1";
    recordWatcher.maxTimestamp = "0";
    
    recordWatcher.minYMDHMS = "";
    recordWatcher.maxYMDHMS = "";
    
    recordWatcher.channel = "";
    
    recordWatcher.unterminatedOnly = false;
    
    recordWatcher.items = new Array();
    
    recordWatcher.__defineGetter__("min", function() {
	return this.minYMDHMS;
    });
    recordWatcher.__defineSetter__("min", function(a) {
	this.minYMDHMS = a;
	this.minTimestamp = timestampToEpoch(a);
    });
    
    recordWatcher.__defineGetter__("max", function() {
	return this.maxYMDHMS;
    });
    recordWatcher.__defineSetter__("max", function(a) {
	this.maxYMDHMS = a;
	this.maxTimestamp = timestampToEpoch(a);
    });
    
    
    recordWatcher.reportUpdate = false;
    recordWatcher.reportTo = this;
    recordWatcher.setFilter(function(item) {
	// item.end >= min
	// item.start <= max
	if (this.unterminatedOnly && ((item.state != "pending") && (item.state != "running"))) {
	    return false;
	}
	
	return 	(item.duration > 0) && 
		(item.state != "canceled") &&
		(item.channel == this.channel) && 
		(cmpEpoch(item.start_time, 0, this.maxTimestamp) <= 0) &&
		(cmpEpoch(item.start_time, item.duration - 1, this.minTimestamp) >= 0);
    });
    
    
    
    recordWatcher.addRecord = function(item) {
	for(i = 0; i < this.items.length; ++i) {
	    if (this.items[i] == item) {
		this.updateRecord(item);
		return;
	    }
	}
	this.items.push(item);
	this.updated();
    }
    
    recordWatcher.removeRecord = function(item) {
	for(i = 0; i < this.items.length; ++i) {
	    if (this.items[i] == item) {
		this.items.splice(i,1);
		this.updated();
		return;
	    }
	}
	this.updated();
    }
    
    recordWatcher.updateRecord = function(item) {
	this.updated();
    }
    
    recordWatcher.updated = function() {};
    
    // Find all records which covers [minTs, maxTs[
    recordWatcher.findRecords = function( minTs, maxTs) {
	var i, item;
	var rslt = undefined;
	for(i = 0; i < recordWatcher.items.length; ++i) {
	    var item, t1, t2;
	    item = recordWatcher.items[i];
	    
	    t1 = epochToTimestamp(item.start_time);
	    
	    t2 = epochToTimestamp(addEpoch(item.start_time, item.duration));
	    
	    if ((t1 <= minTs) && (t2 >= maxTs)) {
		if (rslt == undefined) rslt = new Array;
		rslt.push(item);
	    }
	}
	
	return rslt;
    }
    
    return recordWatcher;
}

print("recorder loaded.\n");
startRecordMonitor();
startRecordMonitorWatchdog();


