//
// class ProgramWatcher
//
// This class watch for channel information. A callback is called
// when a channel gets changed
//
// void watch(channid, when)
//	channid is a channel id (in the channel.conf file)
//	when can be : now, 20060715
// void unwatch()
//	
// void updated()
//	this functions gets called every time the information change
//	(on program end, when information becomes available, ...)
//
// void updateRecords()
//	this functions gets called when record programmation has changed on
//	a program of the request
//
// void getProgram()
//	get the first program matching the criteria. May return undefined
//
// void getPrograms()
//	get the list of all programs matching the criteria.
//	returns undefined if nothing found, a dataview otherwisee 
//

var xmltvSyntax = undefined;
var currentXmltv = undefined;

var programWatchers = new Array();

// Name of the xmltv file
var xmlguidePath;

// Object which watch for fs change...
var xmlguideWatcher;

function ProgramWatcher()
{
    this.programWatcherId = undefined;
    
    this.watch=function(channid, when) {
	if (this.programWatcherId != undefined) {
	    this.unwatch();
	}
	
	this.programWatcherId = programWatchers.length;
	programWatchers[programWatchers.length] = this;
	
	this.programWatcherChannId = channid;
	this.programWatcherWhen = when;
    }
    
    this.unwatch = function() {
	if (this.programWatcherId == undefined) return;
	programWatchers[this.programWatcherId] = programWatchers[programWatchers.length - 1];
	programWatchers.pop();
	this.programWatcherId = undefined;
    }

    this.getPrograms = function() {
	var channel;
	var cid, programmes;
	var timestamp;
	var xmltvId;
	
	if (currentXmltv == undefined) return undefined;
	if (config == undefined) return undefined;
	
	// FIXME : use the tag xmltv-id of the channel
	channel = config.findFirst("/channel[id]", this.programWatcherChannId);
	if (channel == undefined) {
	    debug("channel not found : " + this.programWatcherChannId + "\n");
	    return undefined;
	}
	
	xmltvId = channel.get("internal-xmltv-id");
	if (xmltvId == "") {
	    debug("no xmltv info for channel : " + this.programWatcherChannId + "\n");
	    return undefined;
	}
	
	if (this.programWatcherWhen == "now") {
	    timestamp = nowTimestamp();
	} else if (this.programWatcherWhen == "today") {
	    timestamp = nowTimestamp();
	    timestamp = timestamp.substr(0,8);
	    debug("watching today : "+timestamp+"\n");
	} else {
	    timestamp = this.programWatcherWhen;
	}
	
	// We want program which started before end of period
	// And which end after start of period.
	debug("Searching for program where start <= "+timestamp+"X and stop > "+timestamp+"\n");
	programmes = currentXmltv.find("/programme[channel][>=start][<stop]", xmltvId, timestamp+"X", timestamp);
	
	if (programmes.getItemCount() == 0) {
	    return undefined;
	}
	
	programmes.sort("start")
	return programmes;
    }
    
    this.getProgram = function() {
	var programmes = this.getPrograms();
	var id;
	if (programmes == undefined) return undefined;

	if (programmes.getItemCount() == 1) {
	    return programmes.getItem(0);
	}
	
	debug("Warning: ProgramWatcher.getProgramm is ignoring "+ (programmes.getItemCount() -1) + " programs\n");
	// Return the longer program.
	programmes.sort("stop");
	return programmes.getItem(programmes.getItemCount() - 1);
    }
    
    // By default, updated does nothing...
    this.updated = function() {
    }
    
    this.updateRecords = function() {
    }
}

function updateProgramWatcher()
{
    var i;
    var notif = new Array();
    for(i = 0; i < programWatchers.length; ++i) {
	notif.push(programWatchers[i]);
    }
    
    for(i = 0; i < notif.length; ++i) {
	notif[i].updated();
    }
}

function rebuildXmltvChannelsTable()
{
    var channels;
    var xmltvChannels;
    var i,j;
    if (config == undefined) return;
    if (currentXmltv == undefined) return;
    
    channels = config.find("/channel");
    for(i = 0; i < channels.getItemCount(); ++i) {
	var channel;
	var xmltvmode;
	var channelCol,xmltvCol;
	var resultId = "";
	
	channel = channels.getItem(i);
	
	xmltvmode = channel.get("xmltv-mode");
	if (xmltvmode == "id-id") {
	    channelCol="id";
	    xmltvCol="id";
	} else if (xmltvmode == "id-display") {
	    channelCol="id";
	    xmltvCol="display-name";
	} else if (xmltvmode == "display-id") {
	    channelCol="display";
	    xmltvCol="id";
	} else if (xmltvmode == "display-display") {
	    channelCol="display";
	    xmltvCol="display-name";
	} else if (xmltvmode == "manual-id") {
	    channelCol="xmltv-manual";
	    xmltvCol="id";
	} else if (xmltvmode == "manual-display") {
	    channelCol="xmltv-manual";
	    xmltvCol="display-name";
	} else {
	    channelCol = "";
	    xmltvCol = "";
	}
	
	debug("matching "+channelCol+" with "+xmltvCol+"\n");
	if (channelCol != "") {
	    var searchedString = channel.get(channelCol);
	    debug("looking in xmltv where "+ xmltvCol+"=='"+searchedString+"'\n");
	    if (searchedString != "") {
		xmltvChannels = currentXmltv.find("/channel[(i=)"+xmltvCol+"]",searchedString);
		if ((xmltvChannels != undefined) && (xmltvChannels.getItemCount() > 0)) {
		    resultId = xmltvChannels.getItem(0).get("id");
		}
	    }
	}
	debug("associated "+channel.get("display") + " to "+resultId+"\n");
	channel.set("internal-xmltv-id", resultId);
    }
}

function loadXmltv()
{
    xmlguidePath = getUserConfDir() + "guide.xml";
                                   
    if (xmltvSyntax == undefined) {
	return undefined;
    }
    
    xmlguideWatcher = newFileWatcher();
    xmlguideWatcher.onChange = function() { reloadXmltv(); }
    xmlguideWatcher.path = xmlguidePath;
    
    reloadXmltv();
}

function reloadXmltv()
{
    if ((currentXmltv != undefined)&& (currentXmltv != null)) {
	currentXmltv.dispose();
    }
    
    debug("loading : "+xmlguidePath+"\n");
    currentXmltv = xmltvSyntax.loadFile(xmlguidePath);
    
    rebuildXmltvChannelsTable();
    
    if (displayingChannelBrowser) {
	refreshEPGDisplay();
    }
}


// By default, we display now (when entering the program listing. Then it may change when user browse channels)
var displayTime = "now";

// Contains the date/time corresponding to displayTime
var displayTimeValue;

var displayChannel = undefined;

var displayWatcher = new ProgramWatcher();

var displayProgramWatcher = recorder.newTimestampRequest();

var displayingChannelBrowser = false;

displayWatcher.updated = function()
{
    refreshEPGDisplay();
}
    

function clearDisplay()
{
    // FIXME : clear all text or display a message about unexisting channel ?
    programList.clearItems();
    details.title.text = i18n("Information unavailable");
    details.time.text = "";
    details.categories.text = "";
    details.subtitle.text = "";
    details.description.text = "";
}

function setIcon(widget, xmldata, subtableName)
{
}

function setDateLabel()
{
    day.text = timestampToDateString(displayTimeValue);
}

function refreshEPGDisplay()
{
    var programmes, prog;
    var lookupTimestamp;
    
    clearDisplay();

    debug("displayTime is "+displayTime);
    if (displayTime == "now") {
	lookupTimestamp = nowTimestamp();
	displayWatcher.watch(displayChannel, lookupTimestamp.substr(0,8));
	displayProgramWatcher.min = lookupTimestamp.substr(0,8) + "000000";
	displayProgramWatcher.max = lookupTimestamp.substr(0,8) + "235959";
    } else {
	displayWatcher.watch(displayChannel, displayTime.substr(0,8));
	lookupTimestamp = displayTime;
	displayProgramWatcher.min = displayTime.substr(0,8) + "000000";
	displayProgramWatcher.max = displayTime.substr(0,8) + "235959";
    }
    displayProgramWatcher.channel = displayChannel;
    // Record the value of today (or of displayTime, if not "now")
    displayTimeValue = lookupTimestamp;
    
    setDateLabel();
    
    if ((displayChannel == undefined) || (displayChannel == "")) {
	channelWidget.text = i18n("select a channel");
	return;
    }
    
    displayProgramWatcher.updated = function() {};
    displayProgramWatcher.refreshFilter();
    
    channelWidget.text = displayChannel;
    
    programmes = displayWatcher.getPrograms();
    if (programmes == undefined) {
	return;
    }
    debug("watching for timestamp "+lookupTimestamp+"\n");
    for(i = 0; i < programmes.getItemCount(); ++i) {
	prog = programmes.getItem(i);
	programList.addItem(prog);
	if ((prog.get("start") <= lookupTimestamp + "X") && (prog.get("stop") > lookupTimestamp)) {
	    programList.advance(programList.itemCount() - programList.selectedItemId());
	}
    }

    // FIXME : make the browser scroll at maximum (to view the futur, not the past !)
    if (displayTime == "now") {
	programList.topScroll();
    }
    
    displayProgramWatcher.updated = function() {
	var i;
	for(i = 0 ; i < programList.itemCount(); ++i) {
	    programList.recreateWidget(i);
	}
    }
}

function createWidgetForProgram(xmlProgramme)
{
    var timestamp;
    var records;
    var nvchild = loadXmlWidget("plugins/zapping/programme.xml", null);
    // Start is in the for YYYYMMDDHHMMSS => extract HH:MM
    timestamp = xmlProgramme.get("start");
    nvchild.hour.text = timestamp.substring(8,10) + ":" + timestamp.substring(10,12);
    
    nvchild.title.text = xmlProgramme.get("title");
    
    records = displayProgramWatcher.findRecords(timestamp.substr(0,12), xmlProgramme.get("stop").substr(0,12));
    if (records != undefined) {
	if (records.length > 1) {
	    nvchild.icon.background = "icons/video.png";
	} else {
	    nvchild.icon.background = recorder.getRecordIcon(records[0]);
	}
    }
    
    return nvchild;
}

function loadCurrentProgramDetails()
{
    var i, categories, categoryView, timestamp_start, timestamp_stop;
    var allDesc, desc;
    var xmlProgramme = programList.selectedItem();
    
    if (xmlProgramme == undefined) {
	details.title.text = "";
	details.subtitle.text = "";
	details.time.text = "";
	details.categories.text = "";
	details.desc = "";
	return;
    }
    
    debug("loading current pgroam details : start at "+xmlProgramme.get("start")+", stop at "+xmlProgramme.get("stop")+"...\n");
    details.title.text = xmlProgramme.get("title");
    details.subtitle.text = xmlProgramme.get("sub-title");
    
    timestamp_start = xmlProgramme.get("start");
    timestamp_stop = xmlProgramme.get("stop");
    details.time.text = 
    	timestamp_start.substring(8,10) + ":" + timestamp_start.substring(10,12) 
	+ " - " +
	timestamp_stop.substring(8,10) + ":" + timestamp_stop.substring(10,12);
    
    allDesc = xmlProgramme.find("/desc");
    if (allDesc.getItemCount() > 0) {
	desc = allDesc.getItem(allDesc.getItemCount() - 1);
	details.description.text = desc.get("text");
    } else {
	details.description.text = "";
    }
	
	
    categories = "";
    categoryView = xmlProgramme.find("/category");
    for(i = 0; i < categoryView.getItemCount(); ++i) {
	if (i != 0) {
	    categories += ", ";
	}
	categories += categoryView.getItem(i).get("name");
    }
    
    details.categories.text = categories;
}

programList.createWidgetFor = createWidgetForProgram;
programList.onSelectionChanged = "this.selectionChanged()";
programList.selectionChanged = loadCurrentProgramDetails;

xmltvSyntax = loadSyntaxFromFile("plugins/zapping/xmltv.syntax.xml");
debug("################# LOADING XMLTV\n");
loadXmltv();
refreshEPGDisplay();
debug("################# DONE LOADING XMLTV\n");


//
// Functions related to keys handling
//

// This function return true when key handling should go to
// the channel browser (fullscreen page), rather thant to the
// small selector window
function channelProgramBrowserActive() 
{
    if (selector != undefined) {
	return false;
    }
    
    if (!isCurrentScreen(zapping)) {
	return false;
    }
    
    // BEWARE : when idle, this plugin will reject keys... (strange behaviour)
    if ((currentExternalProgram != undefined) && (currentExternalProgram.fullscreen)) {
	return false;
    }
    
    return true;
}

function backKey() {
    mainmenu.leaveScreen(zapping);
}

function timeMove(id) {
    this.programList.takeFocus();
    if (!this.programList.advance(id)) {
	debug("user is at day boundary : move it !\n");
	// Adjust display time to end of last day (if id is negative)
	// or to begining of the next day (id is positive)
	if (id == -1) {
	    // Go back
	    displayTime = displayTimeValue.substr(0,8)+"235959";
	    displayTimeValue = displayTime;
	    debug("adjusted before move : display time is "+displayTime+"\n");
	    dayMove(-1);
	    
	} else {
	    displayTime = displayTimeValue.substr(0,8)+"000000";
	    displayTimeValue = displayTime;
	    debug("adjusted before move : display time is "+displayTime+"\n");
	    dayMove(1);
	}
    }
}

function dayMove(id) {
    var newTimeStamp;
    
    var delta;
    this.programList.takeFocus();;
    
    // Change day of display time (from displayTimeValue)
    
    displayTime = timestampAddDay(displayTimeValue, id);
    refreshEPGDisplay();
}


function channelMove(id) {
    var allChannels;
    var i, currentId;
    
    // Search for current channel in channel list
    allChannels = config.find("/channel");
    
    if (allChannels.getItemCount() == 0) return;
    
    currentId = -1;
    for(i = 0; i < allChannels.getItemCount(); ++i) {
	if (allChannels.getItem(i).get("id") == displayChannel) {
	    currentId = i;
	    break;
	}
    } 
    
    if (currentId == -1) {
	currentId = 0;
    }
    
    currentId = (currentId + allChannels.getItemCount() + id) % allChannels.getItemCount();
    
    displayChannel = allChannels.getItem(currentId).get("id");
    refreshEPGDisplay();
}

function activateBrowserChannel()
{
    var channel;
    if ((displayChannel == undefined) || (displayChannel == "")) return;
    channel = config.findFirst("/channel[id]", displayChannel);
    if (channel == null) return;
    activateChannel(channel);
}


// This gets called on every channel change.
function channelBrowserChannelChangeListener()
{
    debug("channel change in channel browser.\n");
    if (!displayingChannelBrowser) return;
    // FIXME : adjust "on air" icon
}

channelChangedListener.push(channelBrowserChannelChangeListener);

function record_clicked()
{
    var xmlProgram = programList.selectedItem();
    if (xmlProgram == undefined) return;
    
    var timestamp_start, timestamp_stop;
    var premargin, postmargin;
    
    premargin = parseInt(configHash.get("recorder_premargin"), 10);
    if (isNaN(premargin)) {
	premargin = 0;
    }
    
    postmargin = parseInt(configHash.get("recorder_postmargin"), 10);
    if (isNaN(postmargin)) {
	postmargin = 0;
    }
    
    timestamp_stop = timestampAdjust(xmlProgram.get("stop"), 60 * postmargin);
    timestamp_start = timestampAdjust(xmlProgram.get("start"), -60 * premargin);
    
    // FIXME : if timestamp_stop is before now, popup a dialog (you cannot record in the past !)
    
    var request = recorder.newTimestampRequest();
    request.unterminatedOnly = true;
    request.min = timestamp_start;
    request.max = timestamp_stop;
    request.channel = displayChannel;
    
    request.refreshFilter();
    request.forget();
    print("found "+request.items.length + " records between "+timestamp_start+" and " + timestamp_stop + "\n");
    // Is there record already covering the interval
    var totally;
    totally = request.findRecords(timestamp_start, timestamp_stop);
    if (totally != undefined) {
	// We have at least one record already covering the program.
	print("found totally covering : "+totally.length+" records\n");
	// only one ? Fire the editor for the first record
	if (totally.length == 1) {
	    recorder.editRecordDialog(zapping, totally[0].duplicate());
	    return;
	}
	// Else, just popup the record editor
	recorder.selectRecords(request.items);
	mainmenu.enterScreen(recorder);
	return;
    }
    
    if (request.items.length != 0) {
	if (request.items.length == 1) {
	    
	    var nvobj = request.items[0].duplicate();
	    var nvtimestart, nvtimestop;
	    
	    nvtimestart = nvobj.start_time;
	    nvtimestop = addEpoch(nvobj.start_time, nvobj.duration);
	    
	    if (cmpEpoch(nvtimestop, 0, timestampToEpoch(timestamp_stop)) < 0) {
		nvtimestop = timestampToEpoch(timestamp_stop);
	    }
	    
	    if (cmpEpoch(nvtimestart, 0, timestampToEpoch(timestamp_start)) > 0) {
		nvtimestart = timestampToEpoch(timestamp_start);
	    }
	    
	    nvobj.start_time = nvtimestart;
	    nvobj.duration = subEpoch(nvtimestop, nvtimestart);
	    
	    recorder.editRecordDialog(zapping, nvobj);
	    return;
	}
	
	// Else, just popup the record editor
	recorder.selectRecords(request.items);
	mainmenu.enterScreen(recorder);
	return;
    }
    
    var record = recorder.newEmptyRecord();
    record.title = xmlProgram.get("title");
    if ((xmlProgram.get("sub-title") != "") && (xmlProgram.get("sub-title") != undefined)) {
	record.title = record.title + " - " + xmlProgram.get("sub-title");
    }
    
    record.channel = displayChannel;
    
    var startdate = timestampToEpoch(timestamp_start);
    
    record.start_time = startdate;
    
    var enddate = timestampToEpoch(timestamp_stop);
    
    record.duration = subEpoch(enddate, startdate);
    
    recorder.createNewRecordDialog(zapping, record);
}

// This gets called when user enter this page
function activate() {
    var channel;
    
    if (config == null) {
	config_main.popupTVSettings();
	mainmenu.redirect(this, config_main);
	return;
    }
    
    // By default, seek to current program
    displayTime = "now";
    
    // Take the current channel, or the last, or the first
    if (currentChannel == null) {
	if (lastChannel == null) {
	    channel = config.findFirst("/channel");
	} else {
	    channel = lastChannel;
	}
    } else {
	channel = currentChannel;
    }
    if (channel != null) {
	displayChannel = channel.get("id");
    } else {
	displayChannel = undefined;
    }
    displayingChannelBrowser = true;
    refreshEPGDisplay();
}

// This gets called when user leave this page
function desactivate() 
{
    clearDisplay();
    displayingChannelBrowser = false;
}


