var channelsSyntax = loadSyntaxFromFile("plugins/zapping/channels.syntax.xml");
var config;

// Functions which will be called when currentChannel or currentFrequency change
var channelChangedListener = new Array;
                             
var currentChannel = null;
var currentFrequency = null;
var ignoreTuners = null; // Array, not null after switching to a channel failed => try other frequency
var nextChannel = null;
var nextFrequency = null;

var lastChannel = null;

var currentCommand = null;
var syncCommand = null;
var currentHelper = null;
var prepareCommand = null;

var configWatcher;
/***********************************************************************/
/*                                                                     */
/*                        External interface                           */
/*                                                                     */
/***********************************************************************/

// True when it is ok
function switchChannel(channel, frequency)
{
    nextChannel = channel;
    nextFrequency = frequency;
    lastChannel = channel;
    handleEvent("onChannelChange");
    return currentFrequency == frequency;
}

// Return true if ok
function stopTv()
{
    print("stop TV called\n");
    handleEvent("onStop");
    return currentState == state_idle;
}


// Make sure that the text reporting system is visible to user
// For fullscreen with no osd, turn off
// Return true on success
function makeReportVisible()
{
    if (currentState == undefined) return true;
    if (!currentState.hideEngine) return true;
    if (currentFrequency == null) {
	print("makeReportVisible call stopTV.\n");
	return stopTv();
    }
    
    if (producerAvailableForCommand(currentFrequency.father,"report-command")) {
	return true;
    }
    print("makeReportVisible call stopTV.\n");
    return stopTv();
}

function ostboxIsVisible()
{
    if (currentState == undefined) return true;
    return (!currentState.hideEngine);
}

/***********************************************************************/
/*                                                                     */
/*                        TV State handling                            */
/*                                                                     */
/***********************************************************************/

// State have the following properties :
//
// Mandatory :
//    - name : for debug, mainly
//    - hideEngine : true if ostbox is not visible in this state
//
// Optionals handling event functions :
//    - onStop : when user request tv stop
//    - onChannelChange : when a channel change request occur
//    - onPlayerTerminated : when the player (internal or not) terminates
//

var currentState = undefined;

function handleEvent(event)
{
    print("handleEvent : "+event+"\n");
    if (currentState == undefined) return;
    if (!event in currentState) return;
    if (currentState[event] == undefined) return;
    currentState[event]();
}

function setState(state)
{
    currentState = state;
    print("enter state : "+state.name+"\n");
    print("currentFrequency is : "+ (currentFrequency == null ? null : currentFrequency.get("id")) + "\n");
    print("currentTuner     is : "+ (currentFrequency == null ? null : currentFrequency.father.get("id")) + "\n");
    handleEvent("onEnter");
}

var state_idle = new Object();

state_idle.name = "idle";

state_idle.hideEngine = false;

state_idle.onEnter = function()
{
    runProgram(undefined);

    currentChannel = null;
    currentFrequency = null;
    currentTerminating = null;
    ignoreTuners = null;
    
    abortSyncCommand();
    abortHelperConnection();
    abortPrepareCommand();
    
    mainmenu.leaveScreen(tv_main);
}

state_idle.onChannelChange = function()
{
    enterChannel();
}

var state_preparing = new Object();

state_preparing.name = "preparing";
state_preparing.hideEngine = false;

state_preparing.onChannelChange = function()
{
    abortPrepareCommand();
    if (!enterChannel()) {
	setState(state_idle);
	notifyChannelChanged();
    }

}

state_preparing.onStop = function()
{
    abortPrepareCommand();
    setState(state_idle);

    notifyChannelChanged();
}

state_preparing.onPrepared = function()
{
    if ((!enterPreparedChannel())&&(!reenterChannel())) {
	setState(state_idle);
	notifyChannelChanged();
    }
}

var state_syncing = new Object();

state_syncing.name = "syncing";
state_syncing.hideEngine = false;

state_syncing.onChannelChange = function()
{
    abortSyncCommand();
    abortHelperConnection();
    if (!enterChannel()) {
	setState(state_idle);
	notifyChannelChanged();
    }
}

state_syncing.onStop = function()
{
    abortSyncCommand();
    abortHelperConnection();
    setState(state_idle);
    
    notifyChannelChanged();
}

state_syncing.onSync = function()
{
    if (!enterTunedChannel()) {
	abortSyncCommand();
	abortHelperConnection();
	setState(state_idle);
	notifyChannelChanged();
    }
}

var state_foregroundChildPlaying = new Object();

state_foregroundChildPlaying.name = "foregroundChildPlaying";

state_foregroundChildPlaying.hideEngine = true;

function startForegroundChild()
{
    var frequency = currentFrequency;
    var channel = currentChannel;
    var tuner = currentFrequency.father;
    var ep = new ExternalProgram();
    
    ep.producer = createProducerForCommand(frequency, tuner, channel, "start-command", "");
    ep.interruptible = true;
    ep.fullscreen = true;
    ep.takeAudio = true;
    ep.controlLevel = 1;
    ep.interrupt = function() {
	print("forgroundChild.interrupt call stopTv\n");
	stopTv();
	return currentState == state_idle;
    }
    
    ep.interrupted = function() {
	// Does nothing (this get called just after interrupt)
    }
    
    ep.terminated = function() {
	handleEvent("onPlayerTerminated");
    }
    
    if (!runProgram(ep)) {
	ep.producer.cancel();
	return false;
    }
    
    var reportTextFunction = function(text) {
	if (!producerAvailableForCommand(currentFrequency.father,"report-command")) return;
	abortControlCommand();
	prod = createProducerForCommand(currentFrequency, currentFrequency.father, currentChannel, "report-command", text); 
	prod.start();
	currentCommand = prod;
    }
    
    front.setReportTextFunction(reportTextFunction);
    
    setState(state_foregroundChildPlaying);
    return true;
}

state_foregroundChildPlaying.onChannelChange = function()
{
    var fullScreenHack = false;
    abortControlCommand();
    
    if (!interruptCurrentExternalProgram()) {
	print("failed to interrupt external program !\n");
	return;
    }
    
    front.setReportTextFunction(undefined);

    if (!enterChannel()) {
	setState(state_idle);
	notifyChannelChanged();
    }
}

state_foregroundChildPlaying.onStop = function()
{
    if (!interruptCurrentExternalProgram()) {
	print("failed to interrupt external program !\n");
	return;
    }
    front.setReportTextFunction(undefined);
    
    abortControlCommand();
    
    setState(state_idle);
    
    notifyChannelChanged();
}

state_foregroundChildPlaying.onPlayerTerminated = function()
{
    front.setReportTextFunction(undefined);
    
    abortControlCommand();
    
    setState(state_idle);

    notifyChannelChanged();
}



var state_backgroundChildPlaying = new Object();

state_backgroundChildPlaying.name = "backgroundChildPlaying";

state_backgroundChildPlaying.hideEngine = false;

function startBackgroundChild()
{
    var frequency = currentFrequency;
    var channel = currentChannel;
    var tuner = currentFrequency.father;
    var ep = new ExternalProgram();
    
    ep.producer = createProducerForCommand(frequency, tuner, channel, "start-command", "");
    ep.interruptible = true;
    ep.fullscreen = false;
    ep.takeAudio = true;
    ep.controlLevel = 0;
    ep.interrupt = function() {
	print("backroundChild.interrupt. call stopTv\n");
	stopTv();
	return currentState == state_idle;
    }
    
    ep.interrupted = function() {
	// Does nothing (this get called just after interrupt)
    }
    
    ep.terminated = function() {
	handleEvent("onPlayerTerminated");
    }
    
    if (!runProgram(ep)) {
	ep.producer.cancel();
	return false;
    }
    setState(state_backgroundChildPlaying);
    return true;
}

state_backgroundChildPlaying.onChannelChange = function()
{
    abortControlCommand();
    if (!interruptCurrentExternalProgram()) return;
    
    if (!enterChannel()) {
	setState(state_idle);
	notifyChannelChanged();
    }
}

state_backgroundChildPlaying.onStop = function()
{
    abortControlCommand();
    if (!interruptCurrentExternalProgram()) return;
    
    setState(state_idle);
    notifyChannelChanged();
}

state_backgroundChildPlaying.onPlayerTerminated = function() 
{
    abortControlCommand();
    setState(state_idle);
    notifyChannelChanged();
}



var state_internalVideoPlaying = new Object();

state_internalVideoPlaying.name = "internalVideoPlaying";

state_internalVideoPlaying.hideEngine = false;

// Start playing on a viewer of type internal
function startInternal(withVideo)
{
    var mrl;
    
    tv_main.reportTo = new Object();
    
    tv_main.reportTo.pageEntered = function () { }

    tv_main.reportTo.pageLeaved = function () { print("wrong function called\n"); }
    
    if (withVideo) {
	print("!!!!! overwriting with video !\n");
	tv_main.reportTo.pageLeaved = function() {
	    print("pageLeaved calls stopTv\n");
	    stopTv();
	}
    } else {
	print("!!!!! not overwriting with video !\n");
    }
    
    tv_main.reportTo.mediaTerminated = function() {
	// Nothing to do...
	print("mediaTermianted calls stopTv\n");
	stopTv();
    }
    
    var frequency, channel, tuner;
    frequency = currentFrequency;
    channel = currentChannel;
    tuner = currentFrequency.father;
    print("computing mrl from : "+tuner.get("jsmrl")+"\n");
    mrl = eval(tuner.get("jsmrl"));
    
    // FIXME : tell ignore video for audio only
    // FIXME : specific screen for audio (channel title, current time, ...)
    if (!tv_main.open(mrl, withVideo, getOptionForFrequency(frequency, tuner, channel, "player_deinterlace"))) {
	return false;
    }
    print("media "+mrl+" opened \n");
    if (withVideo) {
	setState(state_internalVideoPlaying);
    } else {
	setState(state_internalAudioPlaying);
    }
    
    // Show the selector
    selectorLoadAll(currentChannel);
    selector.lastNumberTime = undefined;
    selectorSetPossible(0);
    
    return true;
}

function startInternalVideo()
{
    return startInternal(true);
}

state_internalVideoPlaying.onStop = function()
{
    var dummyFunc = function() {
	return true;
    }
    
    tv_main.reportTo = undefined;
    tv_main.stop();
    terminateCurrentProgram(dummyFunc);
    
    setState(state_idle);
    
    notifyChannelChanged();
}

state_internalVideoPlaying.onChannelChange = function()
{
    var dummyFunc = function() {
	return true;
    }
    
    tv_main.reportTo = undefined;
    tv_main.stop();
    terminateCurrentProgram(dummyFunc);
    
    if (!enterChannel()) {
	setState(state_idle);
	notifyChannelChanged();
    }
}

var state_internalAudioPlaying = new Object();

state_internalAudioPlaying.name = "internalVideoPlaying";

state_internalAudioPlaying.hideEngine = false;

function startInternalAudio()
{
    startInternal(false);
}

state_internalAudioPlaying.onStop = function()
{
    var dummyFunc = function() {
	return true;
    }
    
    tv_main.reportTo = undefined;
    tv_main.stop();
    terminateCurrentProgram(dummyFunc);
    
    setState(state_idle);
    
    notifyChannelChanged();
}

state_internalAudioPlaying.onChannelChange = function()
{
    var dummyFunc = function() {
	return true;
    }
    
    tv_main.reportTo = undefined;
    tv_main.stop();
    terminateCurrentProgram(dummyFunc);
    
    if (!enterChannel()) {
	setState(state_idle);
	notifyChannelChanged();
    }
}

// When successfull, enterChannel must start an externalProgram
function enterChannel()
{
    currentChannel = nextChannel;
    currentFrequency = nextFrequency;
    ignoreTuners = null;
    
    print("enterChannel\n");
    mainmenu.enterScreen(tv_main);
    
    return enterUnpreparedChannel() || reenterChannel();
}

function enterUnpreparedChannel()
{
    if (producerAvailableForCommand(currentFrequency.father, "check-command")) {
	prepareCommand = createProducerForCommand(currentFrequency, currentFrequency.father, currentChannel, "check-command");
	prepareCommand.debugFlag = true;
	prepareCommand.childExited = function(code) {
	    prepareCommand = null;
	    this.childExited = undefined;
	    print("check command terminated with code " + code + "\n");
	    if (code == 0) {
		handleEvent("onPrepared");
	    } else if (!reenterChannel()) {
		setState(state_idle);
		notifyChannelChanged();
	    }
	}

	tv_main.reportTo = new Object();
	tv_main.reportTo.pageEntered = function() {}
	tv_main.reportTo.pageLeaved = function() { stopTv(); }

	prepareCommand.start();
	print("check command started\n");
	setState(state_preparing);
	notifyChannelChanged();
	
	return true;
    }

    return enterPreparedChannel();
}

// This gets called from any state except idle
// This will switch to another frequency, not in the ignore array
function reenterChannel()
{
    var frequency, i;
    if (ignoreTuners == null) {
	ignoreTuners = new Array();
    }
    do {
	print("remove tuner " + currentFrequency.father.get("id") + " from possible list\n");
	ignoreTuners.push(currentFrequency.father.get("id"));
	for(i = 0; i < ignoreTuners.length; ++i) {
	    print("ignore list "+i+" is "+ignoreTuners[i]+"\n");
	}
	frequency = findFrequencyForChannel(currentChannel, ignoreTuners);
	if (frequency == null) {
	    print("no more frequencies available. abort.\n");
	    displayUnavailable(currentChannel);
	    return false;
	}
	print("trying frequency on " + currentFrequency.father.get("id") + "\n");
	currentFrequency = frequency;
    } while (!enterUnpreparedChannel());
    
    return true;
}

function helperFifoPath()
{
    if (!currentHelper) return "";
    return currentHelper.fifoPath;
}

// Returns list of records which prevent use of the channel chid
function getRecordsFromHelperProgramList(text, freqid)
{
    var i, count;
    var xmllist, xmlitems, id;
    var arr = new Array();
    
    xmllist = parseXml(text).getRoot();
    
    xmlitems = xmllist.findChilds(xmlNameFilter("client"));
    
    count = 0;
    for(i = 0; i < xmlitems.getItemCount(); ++i) {
	xmlitem = xmlitems.getItem(i);
	
	// Do not care about client on the same frequency
	if (xmlitem.getAttribute("frequency") == freqid) {
	    continue;
	}
	
	id = xmlitem.getAttribute("id");
	if ((id != "") && (id != "live")) {
	    print("client connected : "+id+"\n");
	    arr[count++] = id;
	}
    }
    
    return arr;
}

function enterPreparedChannel()
{
    var helper;
    
    print("enterPreparedChannel\n");
    
    abortSyncCommand();
    abortHelperConnection();
    
    // Check if we have an helper.
    helper = currentFrequency.father.findFirst("/helper");
    if (helper != null) {
	currentHelper = newXmlClient();
	// status of the helper are :
	//    - connecting
	//    - locking
	//    - checking record list
	//    - starting
	//    - unlocking
	//    - playing
	
	// tunerName, 
	
	// Set the server start command here
	currentHelper.setServerAutostartCommand(
			getHelperPath("helpers/media/ostbox-media-helper"),
			currentFrequency.father.get("id") );
	
	currentHelper.fifoPath = createFifo(getUserVarDir()+"tv-fifo");
	
	currentHelper.status = "connecting";
	
	currentHelper.onReply = function() {
	    var rslt;
	    var xml, root;
	    print("got reply from status : "+this.status+"\n");
	    if (this.status == "connecting") {
		this.status = "locking";
		rslt = this.sendXmlRequest("<lock/>", 1000, "locked");
	    } else if (this.status == "locking") {
		this.status = "checking";
		rslt = this.sendXmlRequest("<list/>", 1000, "list_result");
		
	    } else if (this.status == "checking") {
		var clients;
		
		clients = getRecordsFromHelperProgramList(this.lastReply, currentFrequency.get("id"));
		if (clients.length > 0) {
		    print("leave record alone for the moment !\n");
		    this.disconnect();
		    currentHelper = null;
		    if (!reenterChannel()) {
			setState(state_idle);
			notifyChannelChanged();
		    }
		    return;
		}
		
		// Here: check for record
		print(" current list is : "+ this.lastReply + "\n");
		
		xml = parseXml("<start/>");
		root = xml.getRoot();
		root.setAttribute("frequency", currentFrequency.get("id"));
		root.setAttribute("location", this.fifoPath);
		root.setAttribute("id", "live");
		
		var child;
		
		child = helper.father.findFirst("/helper-client-option[name]", "max-initial-delay");
		if (child != null) {
		    root.setAttribute("max-initial-delay", child.get("value"));
		} else {
		    root.setAttribute("max-initial-delay", 6000);
		}
		
		child = helper.father.findFirst("/helper-client-option[name]", "min-initial-data-size");
		if (child != null) {
		       root.setAttribute("min-initial-data-size", child.get("value"));
		} else {
		    root.setAttribute("min-initial-data-size", 4096);
		}
		
		child = helper.father.findFirst("/helper-client-option[name]", "max-live-delay");
		if (child != null) {
		       root.setAttribute("max-live-delay", child.get("value"));
		} else {
		    root.setAttribute("max-live-delay", 6000);
		}
		
		this.status = "starting";
		
		rslt = this.sendXmlRequest(xml.toString(), 1000, "start_result");
	    } else if (this.status == "starting") {
		xml = parseXml(this.lastReply);
		root = xml.getRoot();
		if (root.getAttribute("success") == "1") {
		    this.status = "unlocking";
		    rslt = this.sendXmlRequest("<unlock/>", 1000, "unlocked");
		}
	    } else if (this.status == "unlocking") {
		this.status = "playing";
		handleEvent("onSync");
		rslt = true;
	    } else {
		rslt = false;
	    }
	    if (!rslt) {
		print("error talking to helper : missed transition to "+this.status+"\n");
		// error : disconnect
		this.disconnect();
		currentHelper = null;
		if (!reenterChannel()) {
		    setState(state_idle);
		    notifyChannelChanged();
		}
	    }
	}
	
	currentHelper.onNotification = function() {
	    print("Unhandled notification : "+ this.lastNotificationName+"\n");
	}
	
	currentHelper.onDisconnect = function() {
	    print("got disconnect from status : "+this.status+"\n");
	    currentHelper = null;
	    handleEvent("onStop");
	}
	
	currentHelper.onTimeout = function() {
	    print("got timeout from status : "+this.status+"\n");
	    this.disconnect();
	    currentHelper = null;
	    if (!reenterChannel()) {
		setState(state_idle);
		notifyChannelChanged();
	    }
	}
	
	// connect to the helper. Give 3 seconds for timeout
	if (!currentHelper.connect("127.0.0.1",helper.get("port"), 3000)) {
	    currentHelper = null;
	    return false;
	}

	setState(state_syncing);
	notifyChannelChanged();
	
	return true;
    } 
    
    if (producerAvailableForCommand(currentFrequency.father, "sync-command")) {
	var sync_re;
	syncCommand = createProducerForCommand(currentFrequency, currentFrequency.father, currentChannel, "sync-command");
	syncCommand.debugFlag = true;
	syncCommand.childExited = function() {
	    syncCommand = null;
	    this.childExited = undefined;
	    handleEvent("onStop");
	}

	tv_main.reportTo = new Object();
	tv_main.reportTo.pageEntered = function() {}
	tv_main.reportTo.pageLeaved = function() { stopTv(); }

	sync_re = currentFrequency.father.get("sync-command-ok-re");
	if (sync_re != "") {
	    syncCommand.syncRegExp = new RegExp(sync_re,"i");
	    syncCommand.newEntry = function(str) {
		print("str is "+str+"\n");
		if (this.syncRegExp.test(str)) {
		    handleEvent("onSync");
		}
	    }
	    syncCommand.start();
	    
	    setState(state_syncing);
	    notifyChannelChanged();
	    
	    return true;
	} else {
	    syncCommand.syncRegExp = undefined;
	    syncCommand.start();
	    
	    return enterTunedChannel();
	}
    } else {
	return enterTunedChannel();
    }
}

function enterTunedChannel()
{
    var tuner, result;
    var frequency, channel;
    
    tuner = currentFrequency.father;
    frequency = currentFrequency;
    channel = currentChannel;
    
    if (tuner.get("type") == "foreground-child") {
	// Support for pre-command is for latter.
	result = startForegroundChild();
	notifyChannelChanged();
	return result;
    }
    
    if (tuner.get("type") == "background-child") {
	// Support for pre-command is not for now latter.
	result = startBackgroundChild();
	notifyChannelChanged();
	return result;
    }
    
    if (tuner.get("type") == "internal-video") {
	result = startInternalVideo();
	notifyChannelChanged();
	return result;
    }

    if (tuner.get("type") == "internal-audio") {
	result = startInternalAudio();
	notifyChannelChanged();
	return result;
    }

    return false;
}


/***********************************************************************/
/*                                                                     */
/*                        Helpers functions                            */
/*                                                                     */
/***********************************************************************/

function loadConfig() {
    config = channelsSyntax.loadFile(getUserConfDir() + "channels.xml");
    if (config == null) {
	print("Unable to load file : " + getUserConfDir() + "channels.xml; using default.\n");
    } else {
	debug("Config is\n" + config.asString() +"\n");
    }
}


function producerAvailableForCommand(tuner, command)
{
    return tuner.get(command) != "";
}

function getOptionForFrequency(frequency, tuner, channel, option)
{
    var envview;
    envview = frequency.findFirst("/option[name]", option);
    if (envview == null) {
	envview = frequency.father.findFirst("/option[name]", option);
    }
    if (envview == null) {
	return "";
    }
    return envview.get("value");
}

// tuner is the current tuner 
// channel is the current channel
// frequency is the current frequency
// command is the commad (start-command, stop-command, report-command)
// text is optionnaly the text (only for report-command)
function createProducerForCommand(frequency, tuner, channel, command, text)
{
    var commandline;
    var prod;
    var envview;
    commandline = tuner.get(command);
    debug("starting command "+commandline+"\n");
    prod = newProducer(commandline);
    envview = tuner.find("/"+command+"-env");
    for(i = 0; i <envview.getItemCount(); ++i) {
	var k,v;
	k = envview.getItem(i).get("name");
	v = eval(envview.getItem(i).get("value"));
	prod.setEnv(k, v);
    }
    
    envview = tuner.find("/option");
    for(i = 0; i <envview.getItemCount(); ++i) {
	var k,v;
	k = "OPT_" + envview.getItem(i).get("name");
	v = eval(envview.getItem(i).get("value"));
	prod.setEnv(k, v);
    }
    
    envview = frequency.find("/option");
    for(i = 0; i <envview.getItemCount(); ++i) {
	var k,v;
	k = "OPT_" + envview.getItem(i).get("name");
	v = eval(envview.getItem(i).get("value"));
	prod.setEnv(k, v);
    }
    
    
    return prod;
}

function interruptCurrentExternalProgram()
{
    var killfunc;
    print("interrupting current external program\n");
    // First, try the stop command nicely
    if (producerAvailableForCommand(currentFrequency.father,"stop-command")) {
	var prod;
	prod = createProducerForCommand(currentFrequency, currentFrequency.father, currentChannel, "stop-command", "");
	
	prod.start();
	prod.cancel();
	
	killfunc = function() {
	    currentExternalProgram.producer.discardData();
	    if (currentExternalProgram.producer.completeWithTimeout(graceTime) != -1) return true;
	    // Then kill harder
	    print("kill -15\n");
	    currentExternalProgram.producer.kill(15);
	    if (currentExternalProgram.producer.completeWithTimeout(graceTime) != -1) return true;
	    // And harder
	    print("kill -9\n");
	    currentExternalProgram.producer.kill(9);
	    // Noway but wait...
	    currentExternalProgram.producer.complete();
	    return true;
	}
    } else {
	killfunc = function() {
	    currentExternalProgram.producer.discardData();
	    // Then kill harder first
	    print("kill -15\n");
	    currentExternalProgram.producer.kill(15);
	    if (currentExternalProgram.producer.completeWithTimeout(graceTime) != -1) return true;
	    // And harder
	    print("kill -9\n");
	    currentExternalProgram.producer.kill(9);
	    // Noway but wait...
	    currentExternalProgram.producer.complete();
	    return true;
	}
    }
    
    return terminateCurrentProgram(killfunc);
}

function notifyChannelChanged()
{
    var i,func;
    debug("notifyChannelChanged\n");
    for (i in channelChangedListener) {
	func = channelChangedListener[i];
	func();
    }
}

function abortControlCommand()
{
    if (currentCommand != null) {
	currentCommand.kill(9);
	currentCommand.cancel();
	currentCommand = null;
    }
}

function abortHelperConnection()
{
    if (currentHelper != null) {
	currentHelper.disconnect();
	currentHelper = null;
    }
	
}

function abortSyncCommand()
{
    if (syncCommand != null) {
	syncCommand.discardData();
	syncCommand.childExited = function() {};
	print("kill syncCommand\n");
	syncCommand.kill(15);
	if (syncCommand.completeWithTimeout(graceTime) == -1) {
	    print("command killing harder.\n");
	    syncCommand.kill(9);
	    syncCommand.complete();
	}
	syncCommand = null;
	print("synchro killed.\n");
    }
}

function abortPrepareCommand()
{
    if (prepareCommand != null) {
	prepareCommand.discardData();
	prepareCommand.childExited = function() {};
	print("kill prepareCommand\n");
	prepareCommand.kill(15);
	if (prepareCommand.completeWithTimeout(graceTime) == -1) {
	    print("command killing harder.\n");
	    prepareCommand.kill(9);
	    prepareCommand.complete();
	}
	prepareCommand = null;
	print("prepare commande killed.\n");
    }
}

    

currentState = state_idle;

/***********************************************************************/
/*                                                                     */
/*                         Selector stuffs                             */
/*                                                                     */
/***********************************************************************/
var selector = undefined;
var selectorProgramWatcher = undefined;
// Delay before kill -15 then kill -9
var graceTime = 400;

// En seconde.
var selectorFadeTime = configHash.get("zapping_selectorfadetime");
if (selectorFadeTime == "") selectorFadeTime = 5;
if (selectorFadeTime < 1) selectorFadeTime = 1;

var selectorFadeAutoselect = (configHash.get("zapping_selectorfadeautoselect") == "1");

// En seconde.
var errorFadeTime = selectorFadeTime;

// This gets called when the selector disappear
function selectorElapsed() 
{
    if (selectorFadeAutoselect) {
	go();
    } else {
	selector = undefined;
    }
}


function showDialog() {
    // Stop program in case of fullscreen with no report.
    makeReportVisible();
    
    if (selector == undefined) {
	selector = loadXmlWidget("plugins/zapping/selector.xml", null);
	selector.zapper = this;
	selector.filter = "";
	selector.keyMode = "off";
	selector.lastNumberTime = undefined;
	
	selector.updateTime = function() {
	    var timestamp = nowTimestamp();
	    this.currentTime.text = timestamp.substr(8,2) + ":" + timestamp.substr(10,2);
	}
	
	selector.updateChannelProgram = function() {
	    var prog;
	    if (this.programWatcher == undefined) {
		this.currentProgram.text = "";
		this.progressBar.position = 100;
		return;
	    }
	    
	    prog = this.programWatcher.getProgram();
	    if (prog == undefined) {
		this.currentProgram.text = "";
		this.progressBar.position = 100;
		return;
	    }
	    
	    this.currentProgram.text = prog.get("title");
	    var startDate = timestampToEpoch(prog.get("start"));
	    var endDate = timestampToEpoch(prog.get("stop"));
	    var currentDate = nowEpoch();
	    // Progress is 100 * (currentDate - startDate) / (endDate - startDate)
	    var progress = Math.floor((100.0 * subEpoch(currentDate, startDate)) / subEpoch(endDate, startDate));
	    if (progress < 0) progress = 0;
	    if (progress > 100) progress =  100;
	    debug("startDate is "+startDate+"\n");
	    debug("endDate is "+endDate+"\n");
	    debug("currentDate is "+currentDate+"\n");
	    debug("progress is "+progress+"\n");
	    this.progressBar.position = progress;
	    this.progressBar.refreshLayout();
	}
	
	selector.watchChan = function(id) {
	    if (this.programWatcher == undefined) {
		this.programWatcher = new ProgramWatcher();
		
		this.programWatcher.father = this;
		this.programWatcher.updated = function() {
		    this.father.updateChannelProgram();
		}
	    }
	    this.programWatcher.watch(id, "now");
	    this.updateChannelProgram();
	}
	
	selector.removed = function () {
	    if (this.programWatcher != undefined) {
		this.programWatcher.unwatch();
		this.programWatcher = undefined;
	    }
	    
	    if (selector != this) return;
	    
	    selector = undefined;
	}
	
	selector.elapsed = function () {
	    if (this.programWatcher != undefined) {
		this.programWatcher.unwatch();
		this.programWatcher = undefined;
	    }

	    if (selector != this) return;
	    selectorElapsed();
	}
	
	selector.updateTime();
    }
    
    front.show(selector, front.frequency * selectorFadeTime);
}

function displayUnavailable(channel)
{
    makeReportVisible();
    
    // Display a "not available for the moment"
    var error = loadXmlWidget("plugins/zapping/wrong.xml", null);
    error.textForm = i18n("$1 not available for the moment", channel.get("display"));
    error.textWidget.text = error.textForm;
    selector = undefined;
    front.show(error, front.frequency * errorFadeTime); 
}

// Activate a channel when selector is visible
function go() {
    var chan, frequency;

    if (selector == undefined) return;
    chan = selector.possibleList[selector.possibleId];
    debug("Activing "+chan.get("display")+"\n");

    // Hide the selector
    selector = undefined;
    front.show(undefined, 0);

    activateChannel(chan);
}


// Called to hide the selector
function cancel() {
    if (selector != undefined) {
	selector = undefined;
	front.show(undefined, 0);
    }
}

// Navigate between all possibles item in the selector
function selectorSetPossible(p)
{
    
    var i, shortcut, report;
    
    if (selector == undefined) return;
    
    i = selector.possibleId + p;
    if (i < 0) i += selector.possibleList.length;
    i = i % selector.possibleList.length;
    selector.possibleId = i;
    
    shortcut = selector.possibleList[selector.possibleId].get("shortcut");
    
    debug("shortcut is "+shortcut + " filter is "+selector.filter+"\n");
    if (selector.filter.length > 0) {
	// put ... when shortcut is not complete or multiple possibility exists
	
	if (shortcut.length > selector.filter.length) {
	    selector.shortcutWidget.text = selector.filter+"("+shortcut.substr(selector.filter.length)+")";
	} else {
	    selector.shortcutWidget.text = selector.filter;
	}
	if (selector.possibleList.length > 1) {
	    selector.shortcutWidget.text += "...";
	}
    } else {
	selector.shortcutWidget.text = shortcut;
    }
    
    
    debug("selector.watchChan : "+ selector.possibleList[i].get("id")+"\n");
    selector.watchChan(selector.possibleList[i].get("id"));
    
    selector.channelWidget.text = selector.possibleList[i].get("display");
    if ((selector.possibleList.length > 1) && (selector.filter != "")) {
	selector.textForm = selector.possibleList[i].get("display") + " ?"; // + selector.count.text;
    } else {
	// selector.count.text = "";
    }
    
    report = selector.shortcutWidget.text;
    if (report != "") report += " - ";
    report += selector.channelWidget.text;
    if (selector.currentProgram.text != "") {
	report += "\n" + selector.currentProgram.text;
    }
    /*if (selector.count.text != "") {
	report += " - ";
	report += selector.count.text;
    }*/
    
    selector.textForm = report;
    front.updatedReport();
}

// Load the list of channel corresponding to selector.filter
// Return true if at least one possibility, false otherwiser (and an error window)
function selectorPopulateList()
{
    var cur, i, view, s;
    var possibles;

    cur = selector.filter;
    
    // search the text in shortcuts.
    possibles = new Array();
    view = config.find("/channel");
    for(i = 0; i < view.getItemCount(); ++i) {
	s = view.getItem(i).get("shortcut");
	if (s.substring(0, cur.length) == cur) {
	    debug("got a match : "+s+"\n");
	    possibles[possibles.length] = view.getItem(i);
    	}
    }
    selector.possibleList = possibles;
    selector.possibleId = 0;
    
    if (selector.possibleList == 0) {
	// Nothing matches...
	var error = loadXmlWidget("plugins/zapping/wrong.xml", null);
	error.textForm = i18n("No such channel ($1)", selector.filter);
	error.textWidget.text = error.textForm;
	selector = undefined;
	front.show(error, front.frequency * errorFadeTime); 
	return false;
    }
    
    return true;
}

function selectorLoadAll(channel)
{
    var i;
    showDialog();
    selector.filter = "";
    
    if (!selectorPopulateList()) return;

    if ((channel != null) && (channel != undefined)) {
	for(i = 0; i < selector.possibleList.length; ++i) {
	    if (selector.possibleList[i] == channel) {
		selector.possibleId = i;
		return;
	    }
	}
    }
}

function key(name) 
{
    print("Zapping receive key "+name+"\n");
    var n = timeOfEvent();
    
    if (config == null) return;
    
    showDialog();
    // Two seconds max between two keys
    if ((selector.lastNumberTime != undefined) && (n - selector.lastNumberTime < 2000)) {
	selector.filter = selector.filter + name;
    } else {
	selector.filter = "" + name;
    }
    selector.lastNumberTime = n;
    
    if (!selectorPopulateList()) return;
    
    selectorSetPossible(0);
    if (selector != undefined) {
	front.show(selector, front.frequency * selectorFadeTime); 
    }
}

function change(id) {
    if (selector != undefined) {
	/*if (selector.possibleList.length > 1) {
	    selectorSetPossible(id);
	} else {*/
	    // Iterator throught all channel
	    selectorLoadAll(selector.possibleList[selector.possibleId]);
	    selector.lastNumberTime = undefined;
	    selectorSetPossible(id);
	/*}*/
    } else {
	if (currentChannel != null) {
	    selectorLoadAll(currentChannel);
	    debug("current channel "+selector.possibleId +" is :"+selector.possibleList[selector.possibleId].get("id")+"\n");
	    selector.lastNumberTime = undefined;
	    selectorSetPossible(id);
	} else {
	    selectorLoadAll(lastChannel);
	    selector.lastNumberTime = undefined;
	    selectorSetPossible(0);
	}
    }
    if (selector != undefined) {
	front.show(selector, front.frequency * selectorFadeTime);
    }
}


function activateChannel(chan)
{
    frequency = findFrequencyForChannel(chan);
    
    print("activating : "+chan.get("id")+"\n");
    print("got frequency : "+frequency.get("id")+"\n");
    
    if (frequency == null) {
	displayUnavailable(chan);
	return;
    }
    
    switchChannel(chan, frequency);
}

// Search an available frequency for a channel
// For the moment, return the first found.
// In the futur, may prefer frequency on the current channel, ...
function findFrequencyForChannel(channel, ignoreList)
{
    var i, j;
    var tunerid, frequencyid;
    var ignoreTuner;
    var frequencies, frequency, current;
    var results = new Array();
    frequencies = channel.find("/frequency");
    
    // Honor priority between frequencies
    //frequencies.sort_numeric("priority");
    
    for(i = 0; i < frequencies.getItemCount(); ++i) {
	tunerid = frequencies.getItem(i).get("tuner-id");
	frequencyid = frequencies.getItem(i).get("frequency-id");
	frequency = config.find("/tuner[id]/frequency[id]",tunerid,frequencyid);
	if (ignoreList != null) {
	    ignoreTuner = false;
	    for(j = 0; j < ignoreList.length; ++j) {
		if (ignoreList[j] == tunerid) {
		    ignoreTuner = true;
		}
	    }
	    if (ignoreTuner) {
		continue;
	    }
	}
	
	for(j = 0; j < frequency.getItemCount(); ++j ) {
	    current = frequency.getItem(j);
	    /*
	    if (j == 0 && current.father.get("check-command") != "") {
		// Check that the tuner is available
		var tmpprod = createProducerForCommand(current, current.father, channel, "check-command", "");
		var retcode;
		tmpprod.retcode = 123456789;
		tmpprod.childExited = function(code) {
		    this.retcode = code;
		}
		tmpprod.start();
		tmpprod.complete();
		if (tmpprod.retcode == 123456789) {
		    debug("tmpprod.retcode not updated... : arg !!!\n");
	    	}
		if (tmpprod.retcode != 0) {
		    debug("tuner "+tunerid +" not available at the moment\n");
		    // Tuner not available for the moment
		    break;
		}
	    }*/
	    results[results.length] = current;
	}
    }
    
    if (results.length == 0) {
	return null;
    }
    
    // We have multiple channels...
    // Honor the preferred-tuner
    if (currentFrequency != null) {
	
	debug("Choosing channel while running...\n");
	
	if (currentChannel == channel) {
	    var i, pos;
	    debug("Trying to alternate between "+results.length+"tuners\n");
	    // Iterates throught available frequencies
	    pos = -1;
	    for(i = 0; i < results.length; ++i) {
		if (results[i] == currentFrequency) {
		    pos = i;
		    debug("current tuner is "+results[pos].father.get("id")+"\n");
		    break;
		}
	    }
	    if (pos != -1) {
		pos = pos + 1;
		if (pos == results.length) pos -= results.length;
		debug("now using "+results[pos].father.get("id")+"\n");
		return results[pos];
	    }
	}
	
	if (currentFrequency.father.get("sticky") == "true") {
	    debug("Trying to honor the sticky flag\n");
	    for(i = 0; i < results.length; ++i) {
		if (results[i].father == currentFrequency.father) {
		    debug("honoring sticky flag\n");
		    return results[i];
		}
	    }
	}
    } 
    
    // Just return the first channel
    return results[0];
}

function isOn()
{
    return (currentState != state_idle)
}

function switchPower()
{
    power(!isOn());
}

function power(setOn)
{
    if (isOn() == setOn) {
	return;
    }
    
    
    if (!setOn) {
	stopTv();
	return;
    }
    
    if (lastChannel == null) {
	view = config.find("/channel");
	if (view.getItemCount() > 0) lastChannel = view.getItem(0);
    }
    if (lastChannel != null) {
	frequency = findFrequencyForChannel(lastChannel);
	
	if (frequency != null) {
	    switchChannel(lastChannel, frequency);
	} else {
	    displayUnavailable(lastChannel);
	    nextChannel = null;
	    nextFrequency = null;
	}
    }
}

function reloadConfig()
{
    print("reloading config.\n");
    // Make sure the TV is stopped
    if (!stopTv()) {
	return false;
    }
    
    // Make sure the selector is hidden
    cancel();
    
    currentChannel = null;
    currentFrequency = null;
    ignoreTuners = null;
    nextChannel = null;
    nextFrequency = null;
    lastChannel = null;
    
    loadConfig();
    
    // Update xmltv stuffs
    rebuildXmltvChannelsTable();
    updateProgramWatcher();
    
    // FIXME : ask reloading for all media helper
    // Stop old media helpers
    // => must implement a new command : 
    
    return true;
}


configWatcher = newFileWatcher();
configWatcher.onChange = function() { reloadConfig(); }
configWatcher.path = getUserConfDir() + "channels.xml";


loadConfig();


grabFunction("menu:back", 10);
grabFunction("channel:select", 1);
grabFunction("channel:power", 1);
grabFunction("channel:next", 1);
grabFunction("channel:prev", 1);
grabFunction("channel:0", 1);
grabFunction("channel:1", 1);
grabFunction("channel:2", 1);
grabFunction("channel:3", 1);
grabFunction("channel:4", 1);
grabFunction("channel:5", 1);
grabFunction("channel:6", 1);
grabFunction("channel:7", 1);
grabFunction("channel:8", 1);
grabFunction("channel:9", 1);


