#!/usr/bin/perl -w

eval 'exec /usr/bin/perl -w -S $0 ${1+"$@"}'
    if 0; # not running under some shell
# $Id: get_artist_icon.pl,v 1.1.1.1 2005-11-30 22:25:22 ludovic Exp $

=head1 NAME

get_artist_icon - Get artist image from allmusic.com.

=head1 SYNOPSIS
To download pictures : get_artist_icon [--help] [--artist=ARTIST_NAME] [--artpath=PATH_TO_ARTISTS_FOLDER N]

=head1 DESCRIPTION

Download picture for artist from allmusic.com.

=head1 AUTHOR

Sylvain Fabre, centraladmin@lahiette.com

=cut

# Use Term::ProgressBar if installed.
use constant Have_bar => eval { require Term::ProgressBar; 1 };

use warnings;
use strict;
use Getopt::Long;
use LWP::Simple ();
use LWP::UserAgent;
use HTML::Tree;
use HTML::TreeBuilder;
use HTML::Entities; # parse entities
use IO::File;
use URI;
use Date::Manip;

#***************************************************************************
# Main declarations
#***************************************************************************
my $SEARCH_BASE_URL = 'http://www.allmusic.com/cg/amg.dll';
my $HOME_BASE_URL   = 'http://www.allmusic.com/';
my $MAX_RETRY = 3;
my $ARTIST_DEFAULT_FILENAME = 'artist.jpg';
my $ARTIST_DEFAULT_FILENAME_GIF = 'artist.gif';
my $outfile = "";
sub get_artist_pic( $ );
sub usage();

#***************************************************************************
# Options processing and global variables allocation
#***************************************************************************
my ($opt_artist,  $opt_apath, $opt_help, $opt_default, $opt_summary);
GetOptions('artist=s'       => \$opt_artist,
           'artpath=s'      => \$opt_apath,
           'default=s'      => \$opt_default,
           'summary'        => \$opt_summary,
           'help'           => \$opt_help )
  or usage();
usage() if $opt_help;
if ( !$opt_artist && !$opt_apath ) {
    print "Bad usage ... \n";
    usage();
}
if ($opt_default) {
    $ARTIST_DEFAULT_FILENAME = $opt_default;
}

# Set the user agent for page download
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.0.3705)'); # Identify as MSIE to avoid grabber detection at original website ...
$ua->timeout(30);
$ua->env_proxy;

#***************************************************************************
# Sub sections
#***************************************************************************
sub usage( ) {
print <<END
$0: Download artist picture from AllMusic.com
Summary :
\t$0 --artist=ARTIST_NAME [--default DEFAULT_FILE_NAME]
\tOR
\t$0 --artists_path=PATH_TO_ARTISTS_FOLDER [--default DEFAULT_FILE_NAME]
To download the picture of an artist : $0 --artist=ARTIST_NAME
To recursively parse folders named like artist names : $0 --artists_path=PATH_TO_ARTISTS_FOLDER
If option --default is not specified, then the picture is saved with the filename 'artist.jpg'.
END
  ;
exit;
}
sub search_page ( $ ){
    my $artist = shift;
    my $retry = 0;
    my $response;

    print "Searching now for artist : $artist \n";

    while ( $retry < $MAX_RETRY ) {
        $response = $ua->post($SEARCH_BASE_URL, { 'p' => 'amg', 'sql' => $artist, 'opt1' => '1' } );
        if ($response->is_success) {
            return $response->content;
        } else {
            $retry++;
        }
    }
    die $response->status_line;
}
sub get_page ( $ ){
    my $url = shift;
    my $retry = 0;
    my $response;

    while ( $retry < $MAX_RETRY ) {
        $response = $ua->get($url );
        if ($response->is_success) {
            return $response->content;
        } else {
            $retry++;
        }
    }
    die $response->status_line;
}

if ( $opt_artist ) {
    $outfile = $ARTIST_DEFAULT_FILENAME;
    get_artist_pic($opt_artist);
}

if ( $opt_apath ) {
    opendir DIR, $opt_apath || die "Unable to open the requested directory";
    if ( $opt_summary ) {
        print "Missing pictures for artists : \n";
    }
    while( my $filename = readdir( DIR ) )  {
        next if ( $filename eq '.' || $filename eq '..' );
        if ( opendir (SUBDIR, $opt_apath.$filename) ) {
            $outfile = $opt_apath.$filename.'/'.$ARTIST_DEFAULT_FILENAME;
            next if ( open FILE1, $outfile );
            close FILE1;
            $outfile = $opt_apath.$filename.'/'.$ARTIST_DEFAULT_FILENAME_GIF;
            next if ( open FILE1, $outfile );
            close FILE1;
            if ( $opt_summary ) {
                print "  " . $filename . "\n";
            } else {
                get_artist_pic($filename);
            }
	    closedir SUBDIR;
        }
    }
    closedir(DIR);
}

sub get_picture( $ ) {
    my $url = shift;

    print "Found picture !!!! " . $url ."\n";
    my $response = $ua->get($url, ':content_file' => $outfile );
    if ($response->is_success) {
        print "File saved in " . $outfile . "\n";
    } else {
        print "Error when downloading :".  $response->status_line ."\n";
    }
}

sub check_if_artist_page( $ ) {
    my $tree = shift;

   # Now parse it until
    my $artist_pic = $tree->look_down( '_tag', 'img', 'id' => 'Picture' );
    if ($artist_pic) {
        get_picture($artist_pic->attr('src'));
        return 1;
    } else {
        my $artist_pic2 = $tree->look_down( '_tag', 'script', 'src' => '/js/pictures.js' );
        if ($artist_pic2) {
            print ">>>>>>>>>>>>>>>>>>>> ARTIST with X pictures FOUND !!!! \n";
            return 1;
        }
    }
    return 0;
}

sub get_artist_pic( $ ) {

    my $artist = shift;

    # Now gets the page
    my $p = HTML::TreeBuilder->new;
    $p->parse(search_page($artist));
    $p->eof;

    if ( check_if_artist_page( $p ) ) {
        print "PICTURE FOUND, pass 1\n";
    } else {
        my $count = 1;
        my $baseurl;
        my %artist_tab;
        my %url_tab;
        $artist_tab{$count} = "No search for this artist\n";
        print "Enter 0 to avoid searching for this artist\n";
        # Get the script in the header, because the base url for result is hidden in javascript
        # Now get the list of found artist
        foreach my $cell ( $p->look_down('_tag', 'TD', 'class', 'cell', 'style', 'width:190px;word-wrap:break-word;') ) {
            my $style = $p->look_down('_tag', 'TD', 'class', 'cell', 'style', 'width:254px;word-wrap:break-word;')->as_text;
            print "Artist nb $count : " . $cell->look_down('_tag', 'a')->as_text . " ( " . $style . " )\n";
            $artist_tab{$count} = $cell->look_down('_tag', 'a')->as_text;
            $url_tab{$count} = $cell->look_down('_tag', 'a')->attr('href');
            $count += 1;
        }
        if ( $count > 1 ) {
            print "Choose the artist number : ";
            my $line = <STDIN>;
            if ( eval($line) > 0 ) {
                print "You have choosen artist: " . $artist_tab{eval($line)} . "\n";
                my $p2 = HTML::TreeBuilder->new;
                $p2->parse(get_page($HOME_BASE_URL.$url_tab{eval($line)}));
                $p2->eof;
                # Now parse it again
                if ( check_if_artist_page( $p2 ) ) {
                    print "PICTURE FOUND, pass 2\n";
                } else {
                    print "Sorry, no picture for this artist !!\n";
                }
            }
        } else {
            print "Unknown artist in ALLMUSIC.COM\n";
        }
    }
}

