#!/usr/bin/perl
use bytes;
use Cwd;
use Cwd 'realpath';
use Fcntl ':mode';
use strict;

my %extensions = (
  'bmp' => 'image',
  'gif' => 'image',
  'jpg' => 'image',
  'jpeg' => 'image',
  'png' => 'image',
  'tga' => 'image',

  'avi' => 'video',
  'mpeg' => 'video',
  'mpg' => 'video',
  'wmv' => 'video',

  'm3u' => 'playlist',
  
  'mp3' => 'sound',
  'ogg' => 'sound',
  'wav' => 'sound',
  'wma' => 'sound' );

my %icons = (
  '' => 'icons/file.png',
  'special' => 'icons/special.png',
  'dir' => 'icons/folder.png',
  'img'	=> 'icons/image.png',
  'sound'=> 'icons/sound.png',
  'playlist'=> 'icons/icon_playlist.svgz.png' );
  
my %viewable = (
  'img' => '1');

# Hash to prevent directory recursion
my %recu_prevent;

sub xmlEncode($) {
    my $c;
    my $txt = shift;
    # FIXME : we assume we have unicode here
    my (@chars) = unpack("U0U*", $txt);
    $txt="";
    foreach $c (@chars) {
	if ($c != 0) {
	    $txt.=pack("U", $c);
	    # $txt .= " $c";
	} else {
	    $txt.="X";
	}
    }
    $txt=~s/(\'|\"|\#|&)/"&#".sprintf("%03d",ord($1)).";"/ge;
    return $txt;
}

sub binXmlEncode($) {
    my $txt = shift;
    $txt=~s/(\'|\"|&|\#|[^a-zA-Z0-9 -\;])/"&#".sprintf("%03d",ord($1)).";"/ge;
    return $txt;
}

sub usage() {
    print STDERR "usage: ls.pl [-R] path\n";
    print STDERR "ls.pl was called with ".join(' ', @ARGV)."\n";
    exit(1);
}

#
# list drectory content
# output is :
# <xml>
# <item display="" icon="" path="" viewable="0/1">
#

if (!scalar(@ARGV)) {
    usage();
}

# Turn on recursion
my $recurse = 0;
my $load_playlist = 0;
my $i = 0;
while ($i + 1 < scalar(@ARGV)) {
    my $arg = $ARGV[$i];
    if ($arg eq "-R") {
	$recurse = 1;
	$i++;
    } elsif ($arg eq "--load-playlist") {
	$load_playlist = 1;
	$i++;
    } else {
	usage();
    }
}

my $arg = $ARGV[$i];


print("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
print("<xml>\n");
if (!$load_playlist) {
    recurseRead($arg);
} else {
    loadPlaylist($arg);
}
print("</xml>\n");

sub lsfile($)
{
    my $path = shift;
    my ($display,$metaext,$icon,$view,$isdir,$channel,$date);
    my $file;
    
    if ($path=~m/\/([^\/]+)$/) {
	$file = $1;
    } else {
	$file = $path;
    }
    
    $metaext = '';
    
    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks);
    
    if (!(($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = lstat($path))) {
	print STDERR "$path : $!\n";
	return;
    }
    
    $display = $file;
    
    $isdir = 0;
    if (S_ISLNK($mode)) {
	# If this link points to a directory already visited, abort
	if (!(($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($path))) {
	    print STDERR "$path : $!\n";
	    return;
	}
    }
    
    if (S_ISDIR($mode)) {
	$metaext = 'dir';
	$isdir = 1;
	# Mark the real
	if ($recurse) {
	    # Donot proceed (no output for directory in recursive mode)
	    recurseRead($path);
	    next;
	}
    } elsif (S_ISREG($mode)) {
	# Regular file
	if ($file=~m/(.*)\.(.+)$/) {
	    my ($tmpdisplay,$tmpext) = ($1,lc($2));
	    if (exists $extensions{$tmpext}) {
		$metaext = $extensions{$tmpext};
		$display = $tmpdisplay;
	    }
	}
    } else {
	$metaext = 'special';
    }
    $icon = $icons{$metaext};
    if (exists($viewable{$metaext})) {
	$view = $viewable{$metaext};
    } else {
	$view = '0';
    }
    if ($file =~m/^(\d\d\d\d)\.(\d\d)\.(\d\d)-(\d\d).(\d\d)-(.*)-(.*).avi$/i) {
	$display=$7;
	$channel=$6;
	$date="$3/$2/$1 $4:$5";
    } else {
	$channel="";
	$date="";
    }
    
    print("<item path='" . binXmlEncode($path) . "' icon='" . xmlEncode($icon) . "'" .
	" display='" . xmlEncode($display) . "' channel='" . xmlEncode($channel) . "'" .
	" date='" . xmlEncode($date) . "' viewable='" . xmlEncode($view) . "'" .
	" isdir='" . xmlEncode($isdir) . "' type='".xmlEncode($metaext)."' />\n"); 
}

sub recurseRead($) {
    my $arg = shift;
    my (@content);
    my ($file);
    my (@newDirectory);
    
    $arg=~s/\/+/\//g;
    if (($arg ne "/") && ($arg!~m/\/$/)) { $arg.="/"; }
    
    my $realpath = realpath($arg);
    
    # Check if it was alreay found.
    if (exists $recu_prevent{$realpath}) {
	next;
    }
    $recu_prevent{$realpath} = 1;
    
    opendir(DIRHANDLER,$arg);
    (@content) = grep { /^[^\.]/ } readdir(DIRHANDLER);
    close(DIRHANDLER);
    
    foreach $file (sort @content) {
	lsfile($arg.$file);
    }
}

sub loadPlaylist($)
{
    my $m3u = shift;
    my $line;
    
    my $rootdir = $m3u;
    if ($rootdir=~m/^(.*\/)[^\/]+$/) {
	$rootdir = $1;
    } else {
	$rootdir = "./";
    }
    
    $rootdir = realpath($rootdir)."/";
    $rootdir=~s/\/+/\//g;
    
    if (!open(FIC, "< $m3u")) {
	print STDERR "Failed to open $m3u : $!\n";
	return;
    }
    
    while($line=<FIC>) {
	if ($line=~m/^#/) { next; }
	
	$line=~s/(\n|\r)+$//g;
	
	if ($line eq "") { next; }
	
	$line=~s/\\/\//g;
	if ($line=~m/^\//) {
	    lsfile($line);
	} else {
	    lsfile(realpath($rootdir.$line));
	}
    }
}
