#!/usr/bin/perl

# We must be root !


# Network interface for wake on lan
# Leave empty if none.
$INET_IFACE_WOL = "eth0";

# Set this to 0 if your system can survive a suspend 
# without needing a VT switch out/in.
# This script assume that ostbox runs on vt7 (as Xorg on
# most linux distribution). If that's not the case, modify
# it bellow.
$VT_SWITCH = 1;



# Hardcoded path here (avoir security warning of perl)
$ENV{PATH}="/bin:/sbin:/usr/bin:/usr/sbin";

sub fic_read($$)
{
    my ($file, $content) = (@_);
    my $rslt = "";
    my $line;
    
    open(FIC, "< $file") || return 0;
    while ($line = <FIC>) {
        $rslt .= $line;
    }  
    close(FIC);
    $$content = $rslt;
}

# Write to a system file. Return false on failure
sub fic_write($$)
{
    my ($file, $content) = (@_);
    open(FIC, "> $file") || return 0;
    print FIC $content || return 0;
    close(FIC);
    return 1;
}

$wakeup = $ARGV[0];
$method = $ARGV[1];

print STDERR "suspend.pl : sleep until $wakeup, using method $method\n";

if (! -f "/proc/acpi/sleep") {
    print STDERR "No /proc/acpi/sleep. ACPI suspend is not supported by your kernel/hardware.\n";
    exit(3);
}

if (($wakeup ne "forever") &&  (! -f "/proc/acpi/alarm")) {
    print STDERR "No /proc/acpi/alarm. Unable to program wakeup. Won't go to sleep in these conditions\n";
    exit(3);
}

# Leave X11 (required in most case)
if ($VT_SWITCH && (system("chvt 1") != 0)) {
    print STDERR "Unable to switch to VT1. Sleep aborted\n";
    exit(3);
}

# Put the ethernet in wake on lan mode
if ($INET_IFACE_WOL ne "") {
    # Activate wol on lan on INET
    system("ethtool -s '$INET_IFACE_WOL' wol g");

    my $content;
    
    if (fic_read("/proc/acpi/wakeup",  \$content)) {
        if ($content=~m/NMAC[^\n]+disabled/s) {
            # Make sure acpi will wake up the board on net magic packet
            fic_write("/proc/acpi/wakeup", "NMAC") || print STDERR "Unable to add NMAC to /proc/acpi/wakeup : $!\n";
        }
    } else {
        printf STDERR "Unable to read /proc/acpi/wakeup : $!\n";
    }
}

# Program the wakeup !
if ($wakeup eq "forever") {
    fic_write("/proc/acpi/alarm", "2000-00-00 00:00:00");
} else {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = gmtime($wakeup);
    my $timestamp = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min, $sec);
    print STDERR "Wakeup at $timestamp\n";
    if (!fic_write("/proc/acpi/alarm", $timestamp)) {
        print STDERR "Can't set ACPI alarm : $!\n";
        exit(3);
    }
}

# Sync the disk...
system("sync");

# Go to sleep
$mode = 3;
if ($method eq "disk") { $mode = 4; }
if (!fic_write("/proc/acpi/sleep", $mode)) {
    print STDERR "Can't write into /proc/acpi/sleep : $!\n";
    # Don't exit, switch back to VT in this case  : exit(1);
}

# Sleep time is over here.

# Return to X11 vt
if ($VT_SWITCH) {
    system("chvt 7");
}


#  Clearly, was woken up by a user action.
if ($wakeup eq "forever") {
    exit 2;
}

# TODO : Check if current date match /proc/acpi/alarm ? unreliable

# We can't really decide ...
exit 1;

