#!/bin/bash

# This scripts adjust and read the mixer
#
# its args can be 
#     1  (increase volume)
#     -1 (lower volume)
#     mute
#     unmute
#     toggle
#
# increase/lower automatically unmute volume
# 
# It returns a single line like :
#    32 on
#    33 off
#
#

# By default : control Master and set Front, Center, Surround, LFE to the same value
MASTER="Master"
SLAVES="Front Center Surround LFE"

# If your sound card does not have a Master entry, you can try these :
#
#MASTER="Front"
#SLAVES="Center Surround LFE"

ARGS="sget $MASTER"

case "$1" in
1)
	ARGS="sset $MASTER 1+ unmute"
	;;
+1)
	ARGS="sset $MASTER 1+ unmute"
	;;
-1)
	ARGS="sset $MASTER 1- unmute"
	;;
toggle)
	ARGS="sset $MASTER toggle"
	;;
mute)
	ARGS="sset $MASTER mute"
	;;
unmute)
	ARGS="sset $MASTER unmute"
	;;
esac
	
RESULT=`amixer $ARGS | tail -1`

# Get the percentage
LEVEL=`echo "$RESULT" | sed -n -e '/[(\d+)%]/ s/^.*\[\([0-9]\+\)%\].*/\1/ p'`

# Get the on/off
STATUS=`echo "$RESULT" | sed -n -e '/[(\d+)%]/ s/^.*\[\(on\|off\)\].*/\1/ p'`

if [ "$SLAVES" != "" ]; then
    for MIXER in $SLAVES; do
	if [ "$STATUS" != "off" ]; then
	    amixer sset $MIXER "$LEVEL"% "$STATUS" > /dev/null 2>&1
	else
    	    amixer sset $MIXER "0" "off" > /dev/null 2>&1
	fi
    done
fi	

echo $LEVEL $STATUS
